@@ -1,21 +1,15 @@
 /*
-Copyright (C) 2013-2014 Draios inc.
 
-This file is part of sysdig.
+Copyright (c) 2013-2018 Draios Inc. dba Sysdig.
 
-sysdig is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
+This file is dual licensed under either the MIT or GPL 2. See MIT.txt
+or GPL2.txt for full copies of the license.
 
-sysdig is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#ifndef PPM_H_
+#define PPM_H_
+
 #include <linux/version.h>
 
 /*
@@ -39,35 +33,13 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #if (LINUX_VERSION_CODE > KERNEL_VERSION(3, 12, 0)) && defined(CONFIG_X86)
 #define CAPTURE_PAGE_FAULTS
 #endif
-#define RW_SNAPLEN 80
 #define RW_SNAPLEN_EVENT 4096
-#define RW_MAX_SNAPLEN (256 * 1024 * 1024)
-#define DPI_LOOKAHED_SIZE 16
+#define DPI_LOOKAHEAD_SIZE 16
 #define PPM_NULL_RDEV MKDEV(1, 3)
 #define PPM_PORT_MYSQL 3306
 #define PPM_PORT_POSTGRES 5432
 #define PPM_PORT_STATSD 8125
 
-/*
- * Global enums
- */
-enum syscall_flags {
-	UF_NONE = 0,
-	UF_USED = (1 << 0),
-	UF_NEVER_DROP = (1 << 1),
-	UF_ALWAYS_DROP = (1 << 2),
-	UF_SIMPLEDRIVER_KEEP = (1 << 3),
-};
-
-/*
- * Global structs
- */
-struct syscall_evt_pair {
-	int flags;
-	enum ppm_event_type enter_event_type;
-	enum ppm_event_type exit_event_type;
-};
-
 /*
  * The ring descriptor.
  * We have one of these for each CPU.
@@ -130,8 +102,6 @@ long ppm_strncpy_from_user(char *to, const char __user *from, unsigned long n);
   #define SYSCALL_TABLE_ID0 0
 #endif
 
-#define SYSCALL_TABLE_SIZE 512
-
 extern const struct syscall_evt_pair g_syscall_table[];
 extern const struct ppm_event_info g_event_info[];
 extern const enum ppm_syscall_code g_syscall_code_routing_table[];
@@ -140,3 +110,5 @@ extern const enum ppm_syscall_code g_syscall_code_routing_table[];
 extern const struct syscall_evt_pair g_syscall_ia32_table[];
 extern const enum ppm_syscall_code g_syscall_ia32_code_routing_table[];
 #endif
+
+#endif /* PPM_H_ */
