@@ -1,19 +1,10 @@
 /*
-Copyright (C) 2013-2014 Draios inc.
 
-This file is part of sysdig.
+Copyright (c) 2013-2018 Draios Inc. dba Sysdig.
 
-sysdig is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
+This file is dual licensed under either the MIT or GPL 2. See MIT.txt
+or GPL2.txt for full copies of the license.
 
-sysdig is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef EVENTS_PUBLIC_H_
@@ -25,12 +16,24 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 
 #ifdef __KERNEL__
 #include <linux/types.h>
+#else
+#include "../userspace/common/sysdig_types.h"
+#endif
+
+/*
+ * Macros for packing in different build environments
+ */
+
+#if defined(_WIN64) || defined(WIN64) || defined(_WIN32) || defined(WIN32)
+#define _packed __pragma(pack(push, 1)); __pragma(pack(pop))
+#else
+#define _packed __attribute__((packed))
 #endif
 
 /*
  * Limits
  */
-#define PPM_MAX_EVENT_PARAMS 20	/* Max number of parameters an event can have */
+#define PPM_MAX_EVENT_PARAMS (1 << 5)	/* Max number of parameters an event can have */
 #define PPM_MAX_PATH_SIZE 256	/* Max size that an event parameter can have in the circular buffer, in bytes */
 #define PPM_MAX_NAME_LEN 32
 
@@ -151,9 +154,9 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #define PPM_CL_ACTIVE (1 << 19)			/* libsinsp-specific flag. Set in the first non-clone event for
 										   this thread. */
 #define PPM_CL_CLONE_NEWUSER (1 << 20)
-#define PPM_CL_PIPE_SRC (1 << 21)			/* libsinsp-specific flag. Set if this thread has been 
+#define PPM_CL_PIPE_SRC (1 << 21)			/* libsinsp-specific flag. Set if this thread has been
 										       detected to be the source in a shell pipe. */
-#define PPM_CL_PIPE_DST (1 << 22)			/* libsinsp-specific flag. Set if this thread has been 
+#define PPM_CL_PIPE_DST (1 << 22)			/* libsinsp-specific flag. Set if this thread has been
 										       detected to be the destination in a shell pipe. */
 #define PPM_CL_CLONE_CHILD_CLEARTID (1 << 23)
 #define PPM_CL_CLONE_CHILD_SETTID (1 << 24)
@@ -262,6 +265,12 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
  */
 #define PPM_AT_REMOVEDIR 0x200
 
+/*
+ * linkat() flags
+ */
+#define PPM_AT_SYMLINK_FOLLOW	0x400
+#define PPM_AT_EMPTY_PATH       0x1000
+
 /*
  * rlimit resources
  */
@@ -848,7 +857,13 @@ enum ppm_event_type {
 	PPME_SYSCALL_UNLINKAT_2_X = 303,
 	PPME_SYSCALL_MKDIRAT_E = 304,
 	PPME_SYSCALL_MKDIRAT_X = 305,
-	PPM_EVENT_MAX = 306
+	PPME_SYSCALL_OPENAT_2_E = 306,
+	PPME_SYSCALL_OPENAT_2_X = 307,
+	PPME_SYSCALL_LINK_2_E = 308,
+	PPME_SYSCALL_LINK_2_X = 309,
+	PPME_SYSCALL_LINKAT_2_E = 310,
+	PPME_SYSCALL_LINKAT_2_X = 311,
+	PPM_EVENT_MAX = 312
 };
 /*@}*/
 
@@ -1173,7 +1188,10 @@ enum ppm_syscall_code {
 	PPM_SC_FINIT_MODULE = 314,
 	PPM_SC_BPF = 315,
 	PPM_SC_SECCOMP = 316,
-	PPM_SC_MAX = 317,
+	PPM_SC_SIGALTSTACK = 317,
+	PPM_SC_GETRANDOM = 318,
+	PPM_SC_FADVISE64 = 319,
+	PPM_SC_MAX = 320,
 };
 
 /*
@@ -1259,7 +1277,11 @@ enum ppm_param_type {
 	PT_CHARBUFARRAY = 35,	/* Pointer to an array of strings, exported by the user events decoder. 64bit. For internal use only. */
 	PT_CHARBUF_PAIR_ARRAY = 36,	/* Pointer to an array of string pairs, exported by the user events decoder. 64bit. For internal use only. */
 	PT_IPV4NET = 37, /* An IPv4 network. */
-	PT_MAX = 38 /* array size */
+	PT_IPV6ADDR = 38, /* A 16 byte raw IPv6 address. */
+	PT_IPV6NET = 39, /* An IPv6 network. */
+	PT_IPADDR = 40,  /* Either an IPv4 or IPv6 address. The length indicates which one it is. */
+	PT_IPNET = 41,  /* Either an IPv4 or IPv6 network. The length indicates which one it is. */
+	PT_MAX = 42 /* array size */
 };
 
 enum ppm_print_format {
@@ -1290,7 +1312,7 @@ struct ppm_param_info {
 	const void *info; /**< If this is a flags parameter, it points to an array of ppm_name_value,
 			       else if this is a dynamic parameter it points to an array of ppm_param_info */
 	uint8_t ninfo; /**< Number of entry in the info array. */
-};
+} _packed;
 
 /*!
   \brief Event information.
@@ -1302,9 +1324,8 @@ struct ppm_event_info {
 	enum ppm_event_category category; /**< Event category, e.g. 'file', 'net', etc. */
 	enum ppm_event_flags flags; /**< flags for this event. */
 	uint32_t nparams; /**< Number of parameter in the params array. */
-	/* XXX this 16 limit comes out of my ass. Determine something that makes sense or use a dynamic array. */
 	struct ppm_param_info params[PPM_MAX_EVENT_PARAMS]; /**< parameters descriptions. */
-};
+} _packed;
 
 #if defined _MSC_VER
 #pragma pack(push)
@@ -1322,6 +1343,7 @@ struct ppm_evt_hdr {
 	uint64_t tid; /* the tid of the thread that generated this event */
 	uint32_t len; /* the event len, including the header */
 	uint16_t type; /* the event type */
+	uint32_t nparams; /* the number of parameters of the event */
 };
 #if defined __sun
 #pragma pack()
@@ -1355,6 +1377,7 @@ struct ppm_evt_hdr {
 #define PPM_IOCTL_SET_SIMPLE_MODE _IO(PPM_IOCTL_MAGIC, 18)
 #define PPM_IOCTL_ENABLE_PAGE_FAULTS _IO(PPM_IOCTL_MAGIC, 19)
 #define PPM_IOCTL_GET_N_TRACEPOINT_HIT _IO(PPM_IOCTL_MAGIC, 20)
+#define PPM_IOCTL_GET_PROBE_VERSION _IO(PPM_IOCTL_MAGIC, 21)
 #endif // CYGWING_AGENT
 
 extern const struct ppm_name_value socket_families[];
@@ -1383,6 +1406,7 @@ extern const struct ppm_name_value semctl_commands[];
 extern const struct ppm_name_value access_flags[];
 extern const struct ppm_name_value pf_flags[];
 extern const struct ppm_name_value unlinkat_flags[];
+extern const struct ppm_name_value linkat_flags[];
 
 extern const struct ppm_param_info ptrace_dynamic_param[];
 extern const struct ppm_param_info bpf_dynamic_param[];
@@ -1411,4 +1435,82 @@ struct ppm_proclist_info {
 	struct ppm_proc_info entries[0];
 };
 
+enum syscall_flags {
+	UF_NONE = 0,
+	UF_USED = (1 << 0),
+	UF_NEVER_DROP = (1 << 1),
+	UF_ALWAYS_DROP = (1 << 2),
+	UF_SIMPLEDRIVER_KEEP = (1 << 3),
+};
+
+struct syscall_evt_pair {
+	int flags;
+	enum ppm_event_type enter_event_type;
+	enum ppm_event_type exit_event_type;
+} _packed;
+
+#define SYSCALL_TABLE_SIZE 512
+
+/*
+ * Filler table-related definitions
+ */
+
+#define PPM_MAX_AUTOFILL_ARGS (1 << 2)
+
+/*
+ * Max size of a parameter in the kernel module is u16, so no point
+ * in going beyond 0xffff. However, in BPF the limit is more stringent
+ * because the entire perf event must fit in u16, so make this
+ * a more conservative 65k so we have some room for the other
+ * parameters in the event. It shouldn't cause issues since typically
+ * snaplen is much lower than this.
+ */
+#define PPM_MAX_ARG_SIZE 65000
+
+struct event_filler_arguments;
+
+#include "ppm_fillers.h"
+
+struct ppm_autofill_arg {
+#define AF_ID_RETVAL -1
+#define AF_ID_USEDEFAULT -2
+	int16_t id;
+	long default_val;
+} _packed;
+
+enum autofill_paramtype {
+	APT_REG,
+	APT_SOCK,
+};
+
+typedef int (*filler_callback) (struct event_filler_arguments *args);
+
+struct ppm_event_entry {
+	filler_callback filler_callback;
+	enum ppm_filler_id filler_id;
+	uint16_t n_autofill_args;
+	enum autofill_paramtype paramtype;
+	struct ppm_autofill_arg autofill_args[PPM_MAX_AUTOFILL_ARGS];
+} _packed;
+
+/*
+ * parse_readv_writev_bufs flags
+ */
+#define PRB_FLAG_PUSH_SIZE	1
+#define PRB_FLAG_PUSH_DATA	2
+#define PRB_FLAG_PUSH_ALL	(PRB_FLAG_PUSH_SIZE | PRB_FLAG_PUSH_DATA)
+#define PRB_FLAG_IS_WRITE	4
+
+/*
+ * Return codes
+ */
+#define PPM_SUCCESS 0
+#define PPM_FAILURE_BUFFER_FULL -1
+#define PPM_FAILURE_INVALID_USER_MEMORY -2
+#define PPM_FAILURE_BUG -3
+#define PPM_SKIP_EVENT -4
+
+#define RW_SNAPLEN 80
+#define RW_MAX_SNAPLEN PPM_MAX_ARG_SIZE
+
 #endif /* EVENTS_PUBLIC_H_ */
