@@ -1,19 +1,20 @@
 /*
-Copyright (C) 2013-2014 Draios inc.
+Copyright (C) 2013-2018 Draios Inc dba Sysdig.
 
 This file is part of sysdig.
 
-sysdig is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-sysdig is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 
-You should have received a copy of the GNU General Public License
-along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 ////////////////////////////////////////////////////////////////////////////
@@ -61,6 +62,12 @@ typedef struct wh_t wh_t;
 //
 #define PF_CLONING 1
 
+//
+// ebpf defs
+//
+#define BPF_PROGS_MAX 128
+#define BPF_MAPS_MAX 32
+
 //
 // The device descriptor
 //
@@ -68,13 +75,32 @@ typedef struct scap_device
 {
 	int m_fd;
 	char* m_buffer;
-	struct ppm_ring_buffer_info* m_bufinfo;
 	uint32_t m_lastreadsize;
 	char* m_sn_next_event; // Pointer to the next event available for scap_next
 	uint32_t m_sn_len; // Number of bytes available in the buffer pointed by m_sn_next_event
-	uint32_t m_read_size; // Number of bytes currently ready to be read in this CPU's ring buffer
+	union
+	{
+		// Anonymous struct with ppm stuff
+		struct
+		{
+			struct ppm_ring_buffer_info* m_bufinfo;
+		};
+		// Anonymous struct with bpf stuff
+		struct
+		{
+			uint64_t m_evt_lost;
+		};
+	};
 }scap_device;
 
+
+typedef struct scap_tid
+{
+	uint64_t tid;
+
+	UT_hash_handle hh; ///< makes this structure hashable
+} scap_tid;
+
 //
 // The open instance handle
 //
@@ -91,6 +117,10 @@ struct scap
 	char* m_file_evt_buf;
 	uint32_t m_last_evt_dump_flags;
 	char m_lasterr[SCAP_LASTERR_SIZE];
+
+	// Used for scap_strerror
+	char m_strerror_buf[SCAP_LASTERR_SIZE];
+
 	scap_threadinfo* m_proclist;
 	scap_threadinfo m_fake_kernel_proc;
 	uint64_t m_evtcnt;
@@ -104,10 +134,33 @@ struct scap
 	bool refresh_proc_table_when_saving;
 	uint32_t m_fd_lookup_limit;
 	uint64_t m_unexpected_block_readsize;
+	uint32_t m_ncpus;
 	// Abstraction layer for windows
 #ifdef CYGWING_AGENT
 	wh_t* m_whh;
 #endif
+	bool m_bpf;
+	// Anonymous struct with bpf stuff
+	struct
+	{
+		int m_bpf_prog_fds[BPF_PROGS_MAX];
+		int m_bpf_prog_cnt;
+		bool m_bpf_fillers[BPF_PROGS_MAX];
+		int m_bpf_event_fd[BPF_PROGS_MAX];
+		int m_bpf_map_fds[BPF_MAPS_MAX];
+		int m_bpf_prog_array_map_idx;
+	};
+
+	// The set of process names that are suppressed
+	char **m_suppressed_comms;
+	uint32_t m_num_suppressed_comms;
+
+	// The active set of threads that are suppressed
+	scap_tid *m_suppressed_tids;
+
+	// The number of events that were skipped due to the comm
+	// matching an entry in m_suppressed_comms.
+	uint64_t m_num_suppressed_evts;
 };
 
 typedef enum ppm_dumper_type
@@ -143,7 +196,7 @@ struct scap_ns_socket_list
 //
 
 // Read the full event buffer for the given processor
-int32_t scap_readbuf(scap_t* handle, uint32_t proc, bool blocking, OUT char** buf, OUT uint32_t* len);
+int32_t scap_readbuf(scap_t* handle, uint32_t proc, OUT char** buf, OUT uint32_t* len);
 // Scan a directory containing process information
 int32_t scap_proc_scan_proc_dir(scap_t* handle, char* procdirname, int parenttid, int tid_to_scan, struct scap_threadinfo** pi, char *error, bool scan_sockets);
 // Remove an entry from the process list by parsing a PPME_PROC_EXIT event
@@ -155,14 +208,14 @@ void scap_proc_free_table(scap_t* handle);
 // Copy the fd table of a process into the one of another process
 // int32_t scap_proc_copy_fd_table(scap_t* handle, scap_threadinfo* dst, scap_threadinfo* src);
 // Internal helper function to output the process table to screen
-void scap_proc_print_info(scap_threadinfo* pi);
+void scap_proc_print_info(scap_t *handle, scap_threadinfo* pi);
 void scap_proc_print_table(scap_t* handle);
 // Free all the state related to a process and delete it from the fd table
 void scap_proc_delete(scap_t* handle, scap_threadinfo* proc);
 // Internal helper function to output the fd table of a process
-void scap_fd_print_table(scap_threadinfo* pi);
+void scap_fd_print_table(scap_t *handle, scap_threadinfo* pi);
 // Internal helper function to output an fd table
-void scap_fd_print_fd_table(scap_fdinfo* fds);
+void scap_fd_print_fd_table(scap_t *handle, scap_fdinfo* fds);
 // Given an event, get the info entry for the process that generated it.
 // NOTE: this is different from scap_event_getprocinfo() because it returns the full event information
 // struct scap_threadinfo* scap_proc_get_from_event(scap_t* handle, scap_evt* e);
@@ -173,18 +226,18 @@ void scap_fd_free_ns_sockets_list(scap_t* handle, struct scap_ns_socket_list** s
 // Free a process' fd table
 void scap_fd_free_proc_fd_table(scap_t* handle, scap_threadinfo* pi);
 // Convert an fd entry's info into a string
-int32_t scap_fd_info_to_string(scap_fdinfo* fdi, OUT char* str, uint32_t strlen);
+int32_t scap_fd_info_to_string(scap_t *handle, scap_fdinfo* fdi, OUT char* str, uint32_t strlen);
 // Calculate the length on disk of an fd entry's info
 uint32_t scap_fd_info_len(scap_fdinfo* fdi);
 // Write the given fd info to disk
-int32_t scap_fd_write_to_disk(scap_t* handle, scap_fdinfo* fdi, scap_dumper_t* dumper);
+int32_t scap_fd_write_to_disk(scap_t* handle, scap_fdinfo* fdi, scap_dumper_t* dumper, uint32_t len);
 // Populate the given fd by reading the info from disk
-uint32_t scap_fd_read_from_disk(scap_t* handle, OUT scap_fdinfo* fdi, OUT size_t* nbytes, gzFile f);
+uint32_t scap_fd_read_from_disk(scap_t* handle, OUT scap_fdinfo* fdi, OUT size_t* nbytes, uint32_t block_type, gzFile f);
 // Parse the headers of a trace file and load the tables
 int32_t scap_read_init(scap_t* handle, gzFile f);
 // Add the file descriptor info pointed by fdi to the fd table for process pi.
 // Note: silently skips if fdi->type is SCAP_FD_UNKNOWN.
-int32_t scap_add_fd_to_proc_table(scap_t* handle, scap_threadinfo* pi, scap_fdinfo* fdi);
+int32_t scap_add_fd_to_proc_table(scap_t* handle, scap_threadinfo* pi, scap_fdinfo* fdi, char *error);
 // Remove the given fd from the process table of the process pointed by pi
 void scap_fd_remove(scap_t* handle, scap_threadinfo* pi, int64_t fd);
 // Read an event from disk
@@ -194,7 +247,7 @@ int32_t scap_fd_scan_fd_dir(scap_t* handle, char * procdir, scap_threadinfo* pi,
 // read tcp or udp sockets from the proc filesystem
 int32_t scap_fd_read_ipv4_sockets_from_proc_fs(scap_t* handle, const char * dir, int l4proto, scap_fdinfo ** sockets);
 // read all sockets and add them to the socket table hashed by their ino
-int32_t scap_fd_read_sockets(scap_t* handle, char* procdir, struct scap_ns_socket_list* sockets);
+int32_t scap_fd_read_sockets(scap_t* handle, char* procdir, struct scap_ns_socket_list* sockets, char *error);
 // prints procs details for a give tid
 void scap_proc_print_proc_by_tid(scap_t* handle, uint64_t tid);
 // Allocate and return the list of interfaces on this system
@@ -208,7 +261,36 @@ void scap_free_userlist(scap_userlist* uhandle);
 
 int32_t scap_fd_post_process_unix_sockets(scap_t* handle, scap_fdinfo* sockets);
 
-int32_t scap_proc_fill_cgroups(struct scap_threadinfo* tinfo, const char* procdirname);
+int32_t scap_proc_fill_cgroups(scap_t *handle, struct scap_threadinfo* tinfo, const char* procdirname);
+
+bool scap_alloc_proclist_info(scap_t* handle, uint32_t n_entries);
+
+// Determine whether or not the provided event should be suppressed,
+// based on its event type and parameters. May update the set of
+// suppressed tids as a side-effect.
+//
+// Returns SCAP_FAILURE if we tried to add the tid to the suppressed
+// tid set, but it could *not* be added, SCAP_SUCCESS otherwise.
+int32_t scap_check_suppressed(scap_t *handle, scap_evt *pevent,
+			      bool *suppressed);
+
+// Possibly add or remove the provided comm, tid combination to the
+// set of suppressed processes. If the ptid is currently in the
+// suppressed set, the tid will always be added to the suppressed
+// set. Otherwise, the tid will be added if the comm matches an entry
+// in suppressed_comms.
+//
+// Sets *suppressed to whether, after this check, the tid is suppressed.
+//
+// Returns SCAP_FAILURE if we tried to add the tid to the suppressed
+// tid set, but it could *not* be added, SCAP_SUCCESS otherwise.
+int32_t scap_update_suppressed(scap_t *handle,
+			       const char *comm,
+			       uint64_t tid, uint64_t ptid,
+			       bool *suppressed);
+
+// Wrapper around strerror using buffer in handle
+const char *scap_strerror(scap_t *handle, int errnum);
 
 //
 // ASSERT implementation
@@ -258,6 +340,13 @@ int32_t scap_proc_fill_cgroups(struct scap_threadinfo* tinfo, const char* procdi
 #define SCAP_DRIVER_PROCINFO_INITIAL_SIZE 7
 #define SCAP_DRIVER_PROCINFO_MAX_SIZE 128000
 
+extern const enum ppm_syscall_code g_syscall_code_routing_table[];
+extern const struct syscall_evt_pair g_syscall_table[];
+extern const struct ppm_event_info g_event_info[];
+extern const struct ppm_syscall_desc g_syscall_info_table[];
+extern const struct ppm_event_entry g_ppm_events[];
+extern bool validate_info_table_size();
+
 #ifdef __cplusplus
 }
 #endif
