@@ -1,19 +1,20 @@
 /*
-Copyright (C) 2013-2014 Draios inc.
+Copyright (C) 2013-2018 Draios Inc dba Sysdig.
 
 This file is part of sysdig.
 
-sysdig is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-sysdig is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 
-You should have received a copy of the GNU General Public License
-along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <stdio.h>
@@ -21,6 +22,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "scap.h"
 #include "scap-int.h"
+#include "scap_savefile.h"
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <fcntl.h>
@@ -32,6 +34,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include <sys/socket.h>
 #include <netdb.h>
 #include <arpa/inet.h>
+#include <errno.h>
 #else
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
@@ -55,27 +58,48 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 
 #define SOCKET_SCAN_BUFFER_SIZE 1024 * 1024
 
-int32_t scap_fd_print_ipv6_socket_info(scap_fdinfo *fdi, OUT char *str, uint32_t stlen)
+int32_t scap_fd_print_ipv6_socket_info(scap_t *handle, scap_fdinfo *fdi, OUT char *str, uint32_t stlen)
 {
 	char source_address[100];
 	char destination_address[100];
 	if(NULL == inet_ntop(AF_INET6,fdi->info.ipv6info.sip,source_address,100))
 	{
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE,
+			 "Could not convert IPv6 source address 0x%x%x%x%x (%s)",
+			 fdi->info.ipv6info.sip[0],
+			 fdi->info.ipv6info.sip[1],
+			 fdi->info.ipv6info.sip[2],
+			 fdi->info.ipv6info.sip[3],
+			 scap_strerror(handle, errno));
 		return SCAP_FAILURE;
 	}
 	if(NULL == inet_ntop(AF_INET6,fdi->info.ipv6info.dip,destination_address,100))
 	{
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE,
+			 "Could not convert IPv6 source address 0x%x%x%x%x (%s)",
+			 fdi->info.ipv6info.dip[0],
+			 fdi->info.ipv6info.dip[1],
+			 fdi->info.ipv6info.dip[2],
+			 fdi->info.ipv6info.dip[3],
+			 scap_strerror(handle, errno));
 		return SCAP_FAILURE;
 	}
 	snprintf(str,stlen,"%s:%u->%s:%u",source_address,fdi->info.ipv6info.sport,destination_address,fdi->info.ipv6info.dport);
 	return SCAP_SUCCESS;
 }
 
-int32_t scap_fd_print_ipv6_server_socket_info(scap_fdinfo *fdi, OUT char *str, uint32_t stlen)
+int32_t scap_fd_print_ipv6_server_socket_info(scap_t *handle, scap_fdinfo *fdi, OUT char *str, uint32_t stlen)
 {
 	char address[100];
 	if(NULL == inet_ntop(AF_INET6,fdi->info.ipv6serverinfo.ip,address,100))
 	{
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE,
+			 "Could not convert IPv6 source address 0x%x%x%x%x (%s)",
+			 fdi->info.ipv6serverinfo.ip[0],
+			 fdi->info.ipv6serverinfo.ip[1],
+			 fdi->info.ipv6serverinfo.ip[2],
+			 fdi->info.ipv6serverinfo.ip[3],
+			 scap_strerror(handle, errno));
 		return SCAP_FAILURE;
 	}
 	snprintf(str,stlen,"%s:%u->:::*",address,fdi->info.ipv6serverinfo.port);
@@ -85,13 +109,14 @@ int32_t scap_fd_print_ipv6_server_socket_info(scap_fdinfo *fdi, OUT char *str, u
 //
 // Convert an fd entry's info into a string
 //
-int32_t scap_fd_info_to_string(scap_fdinfo *fdi, OUT char *str, uint32_t stlen)
+int32_t scap_fd_info_to_string(scap_t *handle, scap_fdinfo *fdi, OUT char *str, uint32_t stlen)
 {
 	//
 	// Input validation
 	//
 	if((fdi)->type == SCAP_FD_UNKNOWN)
 	{
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "fd type unknown");
 		return SCAP_FAILURE;
 	}
 
@@ -119,10 +144,10 @@ int32_t scap_fd_info_to_string(scap_fdinfo *fdi, OUT char *str, uint32_t stlen)
 		         (uint32_t)fdi->info.ipv4serverinfo.port);
 		break;
 	case SCAP_FD_IPV6_SOCK:
-		return scap_fd_print_ipv6_socket_info(fdi,str,stlen);
+		return scap_fd_print_ipv6_socket_info(handle,fdi,str,stlen);
 		break;
 	case SCAP_FD_IPV6_SERVSOCK:
-		return scap_fd_print_ipv6_server_socket_info(fdi,str,stlen);
+		return scap_fd_print_ipv6_server_socket_info(handle,fdi,str,stlen);
 		break;
 	case SCAP_FD_FIFO:
 		snprintf(str, stlen, "<PIPE>");
@@ -157,6 +182,7 @@ int32_t scap_fd_info_to_string(scap_fdinfo *fdi, OUT char *str, uint32_t stlen)
  		break;
 	default:
 		ASSERT(false);
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "fd type unrecognized");
 		return SCAP_FAILURE;
 	}
 
@@ -168,7 +194,11 @@ int32_t scap_fd_info_to_string(scap_fdinfo *fdi, OUT char *str, uint32_t stlen)
 //
 uint32_t scap_fd_info_len(scap_fdinfo *fdi)
 {
-	uint32_t res = sizeof(fdi->ino) + 1 + sizeof(fdi->fd);
+	//
+	// NB: new fields must be appended
+	//
+
+	uint32_t res = sizeof(uint32_t) + sizeof(fdi->ino) + 1 + sizeof(fdi->fd);
 
 	switch(fdi->type)
 	{
@@ -197,8 +227,8 @@ uint32_t scap_fd_info_len(scap_fdinfo *fdi)
 				sizeof(uint8_t); // l4proto
 		break;
 	case SCAP_FD_UNIX_SOCK:
-		res += 
-			sizeof(uint64_t) + // unix source 
+		res +=
+			sizeof(uint64_t) + // unix source
 			sizeof(uint64_t) +  // unix destination
 			(uint32_t)strnlen(fdi->info.unix_socket_info.fname, SCAP_MAX_PATH_SIZE) + 2;
 		break;
@@ -231,12 +261,13 @@ int scap_dump_write(scap_dumper_t *d, void* buf, unsigned len);
 //
 // Write the given fd info to disk
 //
-int32_t scap_fd_write_to_disk(scap_t *handle, scap_fdinfo *fdi, scap_dumper_t *d)
+int32_t scap_fd_write_to_disk(scap_t *handle, scap_fdinfo *fdi, scap_dumper_t *d, uint32_t len)
 {
 
 	uint8_t type = (uint8_t)fdi->type;
 	uint16_t stlen;
-	if(scap_dump_write(d, &(fdi->fd), sizeof(uint64_t)) != sizeof(uint64_t) ||
+	if(scap_dump_write(d, &(len), sizeof(uint32_t)) != sizeof(uint32_t) ||
+	        scap_dump_write(d, &(fdi->fd), sizeof(uint64_t)) != sizeof(uint64_t) ||
 	        scap_dump_write(d, &(fdi->ino), sizeof(uint64_t)) != sizeof(uint64_t) ||
 	        scap_dump_write(d, &(type), sizeof(uint8_t)) != sizeof(uint8_t))
 	{
@@ -383,19 +414,33 @@ uint32_t scap_fd_read_fname_from_disk(scap_t* handle, char* fname,OUT size_t* nb
 // Populate the given fd by reading the info from disk
 // Returns the number of read bytes.
 //
-uint32_t scap_fd_read_from_disk(scap_t *handle, OUT scap_fdinfo *fdi, OUT size_t *nbytes, gzFile f)
+uint32_t scap_fd_read_from_disk(scap_t *handle, OUT scap_fdinfo *fdi, OUT size_t *nbytes, uint32_t block_type, gzFile f)
 {
 	uint8_t type;
+	uint32_t toread;
+	int fseekres;
+	uint32_t sub_len = 0;
 	uint32_t res = SCAP_SUCCESS;
 	*nbytes = 0;
 
-	if(scap_fd_read_prop_from_disk(handle, &(fdi->fd), sizeof(fdi->fd), nbytes, f) ||
+	if((block_type == FDL_BLOCK_TYPE_V2 && scap_fd_read_prop_from_disk(handle, &sub_len, sizeof(uint32_t), nbytes, f)) ||
+	        scap_fd_read_prop_from_disk(handle, &(fdi->fd), sizeof(fdi->fd), nbytes, f) ||
 	        scap_fd_read_prop_from_disk(handle, &(fdi->ino), sizeof(fdi->ino), nbytes, f) ||
 	        scap_fd_read_prop_from_disk(handle, &type, sizeof(uint8_t), nbytes, f))
 	{
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "Could not read prop block for fd");
 		return SCAP_FAILURE;
 	}
 
+	// If new parameters are added, sub_len can be used to
+	// see if they are available in the current capture.
+	// For example, for a 32bit parameter:
+	//
+	// if(sub_len && (*nbytes + sizeof(uint32_t)) <= sub_len)
+	// {
+	//    ...
+	// }
+
 	fdi->type = (scap_fd_type)type;
 
 	switch(fdi->type)
@@ -491,6 +536,26 @@ uint32_t scap_fd_read_from_disk(scap_t *handle, OUT scap_fdinfo *fdi, OUT size_t
 		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "error reading the fd info from file, wrong fd type %u", (uint32_t)fdi->type);
 		return SCAP_FAILURE;
 	}
+
+	if(sub_len && *nbytes != sub_len)
+	{
+		if(*nbytes > sub_len)
+		{
+			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "corrupted input file. Had read %lu bytes, but fdlist entry have length %u.",
+				 *nbytes, sub_len);
+			return SCAP_FAILURE;
+		}
+		toread = sub_len - *nbytes;
+		fseekres = (int)gzseek(f, (long)toread, SEEK_CUR);
+		if(fseekres == -1)
+		{
+			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "corrupted input file. Can't skip %u bytes.",
+				 (unsigned int)toread);
+			return SCAP_FAILURE;
+		}
+		*nbytes = sub_len;
+	}
+
 	return res;
 }
 
@@ -553,7 +618,7 @@ void scap_fd_remove(scap_t *handle, scap_threadinfo *tinfo, int64_t fd)
 		// Looks like there's no fd to remove.
 		// Likely, the fd creation event was dropped.
 		//
-		//scap_proc_print_info(tinfo);
+		//scap_proc_print_info(handle, tinfo);
 		//      ASSERT(false);
 		return;
 	}
@@ -566,7 +631,7 @@ void scap_fd_remove(scap_t *handle, scap_threadinfo *tinfo, int64_t fd)
 // Add the file descriptor info pointed by fdi to the fd table for process tinfo.
 // Note: silently skips if fdi->type is SCAP_FD_UNKNOWN.
 //
-int32_t scap_add_fd_to_proc_table(scap_t *handle, scap_threadinfo *tinfo, scap_fdinfo *fdi)
+int32_t scap_add_fd_to_proc_table(scap_t *handle, scap_threadinfo *tinfo, scap_fdinfo *fdi, char *error)
 {
 	int32_t uth_status = SCAP_SUCCESS;
 	scap_fdinfo *tfdi;
@@ -597,7 +662,7 @@ int32_t scap_add_fd_to_proc_table(scap_t *handle, scap_threadinfo *tinfo, scap_f
 		HASH_ADD_INT64(tinfo->fdlist, fd, fdi);
 		if(uth_status != SCAP_SUCCESS)
 		{
-			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "process table allocation error (2)");
+			snprintf(error, SCAP_LASTERR_SIZE, "process table allocation error (2)");
 			return SCAP_FAILURE;
 		}
 	}
@@ -621,6 +686,8 @@ int32_t scap_fd_handle_pipe(scap_t *handle, char *fname, scap_threadinfo *tinfo,
 	r = readlink(fname, link_name, SCAP_MAX_PATH_SIZE);
 	if (r <= 0)
 	{
+		snprintf(error, SCAP_LASTERR_SIZE, "Could not read link %s (%s)",
+			 fname, scap_strerror(handle, errno));
 		return SCAP_FAILURE;
 	}
 	link_name[r] = '\0';
@@ -637,7 +704,7 @@ int32_t scap_fd_handle_pipe(scap_t *handle, char *fname, scap_threadinfo *tinfo,
 	strncpy(fdi->info.fname, link_name, SCAP_MAX_PATH_SIZE);
 
 	fdi->ino = ino;
-	return scap_add_fd_to_proc_table(handle, tinfo, fdi);
+	return scap_add_fd_to_proc_table(handle, tinfo, fdi, error);
 }
 
 static inline uint32_t open_flags_to_scap(unsigned long flags)
@@ -823,7 +890,7 @@ int32_t scap_fd_handle_regular_file(scap_t *handle, char *fname, scap_threadinfo
 		strncpy(fdi->info.fname, link_name, SCAP_MAX_PATH_SIZE);
 	}
 
-	return scap_add_fd_to_proc_table(handle, tinfo, fdi);
+	return scap_add_fd_to_proc_table(handle, tinfo, fdi, error);
 }
 
 int32_t scap_fd_handle_socket(scap_t *handle, char *fname, scap_threadinfo *tinfo, scap_fdinfo *fdi, char* procdir, uint64_t net_ns, struct scap_ns_socket_list **sockets_by_ns, char *error)
@@ -839,7 +906,7 @@ int32_t scap_fd_handle_socket(scap_t *handle, char *fname, scap_threadinfo *tinf
 	{
 		return SCAP_SUCCESS;
 	}
-	else 
+	else
 	{
 		HASH_FIND_INT64(*sockets_by_ns, &net_ns, sockets);
 		if(sockets == NULL)
@@ -847,16 +914,18 @@ int32_t scap_fd_handle_socket(scap_t *handle, char *fname, scap_threadinfo *tinf
 			sockets = malloc(sizeof(struct scap_ns_socket_list));
 			sockets->net_ns = net_ns;
 			sockets->sockets = NULL;
+			char fd_error[SCAP_LASTERR_SIZE];
 
 			HASH_ADD_INT64(*sockets_by_ns, net_ns, sockets);
 			if(uth_status != SCAP_SUCCESS)
 			{
-				snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "socket list allocation error");
-				return SCAP_FAILURE;				
+				snprintf(error, SCAP_LASTERR_SIZE, "socket list allocation error");
+				return SCAP_FAILURE;
 			}
 
-			if(scap_fd_read_sockets(handle, procdir, sockets) == SCAP_FAILURE)
+			if(scap_fd_read_sockets(handle, procdir, sockets, fd_error) == SCAP_FAILURE)
 			{
+				snprintf(error, SCAP_LASTERR_SIZE, "Cannot read sockets (%s)", fd_error);
 				sockets->sockets = NULL;
 				return SCAP_FAILURE;
 			}
@@ -878,7 +947,7 @@ int32_t scap_fd_handle_socket(scap_t *handle, char *fname, scap_threadinfo *tinf
 	{
 		// it's a kind of socket, but we don't support it right now
 		fdi->type = SCAP_FD_UNSUPPORTED;
-		return scap_add_fd_to_proc_table(handle, tinfo, fdi);
+		return scap_add_fd_to_proc_table(handle, tinfo, fdi, error);
 	}
 
 	//
@@ -890,7 +959,7 @@ int32_t scap_fd_handle_socket(scap_t *handle, char *fname, scap_threadinfo *tinf
 		memcpy(&(fdi->info), &(tfdi->info), sizeof(fdi->info));
 		fdi->ino = ino;
 		fdi->type = tfdi->type;
-		return scap_add_fd_to_proc_table(handle, tinfo, fdi);
+		return scap_add_fd_to_proc_table(handle, tinfo, fdi, error);
 	}
 	else
 	{
@@ -911,6 +980,9 @@ int32_t scap_fd_read_unix_sockets_from_proc_fs(scap_t *handle, const char* filen
 	if(NULL == f)
 	{
 		ASSERT(false);
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "Could not open sockets file %s (%s)",
+			 filename,
+			 scap_strerror(handle, errno));
 		return SCAP_FAILURE;
 	}
 	while(NULL != fgets(line, sizeof(line), f))
@@ -986,7 +1058,7 @@ int32_t scap_fd_read_unix_sockets_from_proc_fs(scap_t *handle, const char* filen
 			free(fdinfo);
 			continue;
 		}
-		
+
 		// 7. Inode
 		token = strtok_r(NULL, delimiters, &scratch);
 		if(token == NULL)
@@ -1013,6 +1085,7 @@ int32_t scap_fd_read_unix_sockets_from_proc_fs(scap_t *handle, const char* filen
 		if(uth_status != SCAP_SUCCESS)
 		{
 			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "unix socket allocation error");
+			fclose(f);
 			return SCAP_FAILURE;
 		}
 	}
@@ -1036,6 +1109,10 @@ int32_t scap_fd_read_netlink_sockets_from_proc_fs(scap_t *handle, const char* fi
 	if(NULL == f)
 	{
 		ASSERT(false);
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "Could not open netlink sockets file %s (%s)",
+			 filename,
+			 scap_strerror(handle, errno));
+
 		return SCAP_FAILURE;
 	}
 	while(NULL != fgets(line, sizeof(line), f))
@@ -1152,6 +1229,7 @@ int32_t scap_fd_read_netlink_sockets_from_proc_fs(scap_t *handle, const char* fi
 		if(uth_status != SCAP_SUCCESS)
 		{
 			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "netlink socket allocation error");
+			fclose(f);
 			return SCAP_FAILURE;
 		}
 	}
@@ -1175,7 +1253,7 @@ int32_t scap_fd_read_ipv4_sockets_from_proc_fs(scap_t *handle, const char *dir,
 	if(scan_buf == NULL)
 	{
 		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "scan_buf allocation error");
-		return SCAP_FAILURE;		
+		return SCAP_FAILURE;
 	}
 
 	f = fopen(dir, "r");
@@ -1183,9 +1261,12 @@ int32_t scap_fd_read_ipv4_sockets_from_proc_fs(scap_t *handle, const char *dir,
 	{
 		ASSERT(false);
 		free(scan_buf);
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "Could not open ipv4 sockets dir %s (%s)",
+			 dir,
+			 scap_strerror(handle, errno));
 		return SCAP_FAILURE;
 	}
-	
+
 	while((rsize = fread(scan_buf, 1, SOCKET_SCAN_BUFFER_SIZE, f))  != 0)
 	{
 		char* scan_end = scan_buf + rsize;
@@ -1201,7 +1282,7 @@ int32_t scap_fd_read_ipv4_sockets_from_proc_fs(scap_t *handle, const char *dir,
 			}
 
 			scap_fdinfo *fdinfo = malloc(sizeof(scap_fdinfo));
-			
+
 			//
 			// Skip the sl field
 			//
@@ -1318,11 +1399,11 @@ int32_t scap_fd_read_ipv4_sockets_from_proc_fs(scap_t *handle, const char *dir,
 			if(uth_status != SCAP_SUCCESS)
 			{
 				uth_status = SCAP_FAILURE;
-				// TODO: set some error message
+				snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "ipv4 socket allocation error");
 				break;
 			}
 
-			scan_pos++;			
+			scan_pos++;
 		}
 	}
 
@@ -1352,7 +1433,7 @@ int32_t scap_fd_read_ipv6_sockets_from_proc_fs(scap_t *handle, char *dir, int l4
 	if(scan_buf == NULL)
 	{
 		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "scan_buf allocation error");
-		return SCAP_FAILURE;		
+		return SCAP_FAILURE;
 	}
 
 	f = fopen(dir, "r");
@@ -1361,6 +1442,9 @@ int32_t scap_fd_read_ipv6_sockets_from_proc_fs(scap_t *handle, char *dir, int l4
 	{
 		ASSERT(false);
 		free(scan_buf);
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "Could not open ipv6 sockets dir %s (%s)",
+			 dir,
+			 scap_strerror(handle, errno));
 		return SCAP_FAILURE;
 	}
 
@@ -1379,7 +1463,7 @@ int32_t scap_fd_read_ipv6_sockets_from_proc_fs(scap_t *handle, char *dir, int l4
 			}
 
 			scap_fdinfo *fdinfo = malloc(sizeof(scap_fdinfo));
-			
+
 			//
 			// Skip the sl field
 			//
@@ -1537,11 +1621,11 @@ int32_t scap_fd_read_ipv6_sockets_from_proc_fs(scap_t *handle, char *dir, int l4
 			if(uth_status != SCAP_SUCCESS)
 			{
 				uth_status = SCAP_FAILURE;
-				// TODO: set some error message
+				snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "ipv6 socket allocation error");
 				break;
 			}
 
-			scan_pos++;			
+			scan_pos++;
 		}
 	}
 
@@ -1551,7 +1635,7 @@ int32_t scap_fd_read_ipv6_sockets_from_proc_fs(scap_t *handle, char *dir, int l4
 	return uth_status;
 }
 
-int32_t scap_fd_read_sockets(scap_t *handle, char* procdir, struct scap_ns_socket_list *sockets)
+int32_t scap_fd_read_sockets(scap_t *handle, char* procdir, struct scap_ns_socket_list *sockets, char *error)
 {
 	char filename[SCAP_MAX_PATH_SIZE];
 	char netroot[SCAP_MAX_PATH_SIZE];
@@ -1575,27 +1659,31 @@ int32_t scap_fd_read_sockets(scap_t *handle, char* procdir, struct scap_ns_socke
 	if(scap_fd_read_ipv4_sockets_from_proc_fs(handle, filename, SCAP_L4_TCP, &sockets->sockets) == SCAP_FAILURE)
 	{
 		scap_fd_free_table(handle, &sockets->sockets);
-		return SCAP_FAILURE;		
+		snprintf(error, SCAP_LASTERR_SIZE, "Could not read ipv4 tcp sockets (%s)", handle->m_lasterr);
+		return SCAP_FAILURE;
 	}
 
 	snprintf(filename, sizeof(filename), "%sudp", netroot);
 	if(scap_fd_read_ipv4_sockets_from_proc_fs(handle, filename, SCAP_L4_UDP, &sockets->sockets) == SCAP_FAILURE)
 	{
 		scap_fd_free_table(handle, &sockets->sockets);
-		return SCAP_FAILURE;		
+		snprintf(error, SCAP_LASTERR_SIZE, "Could not read ipv4 udp sockets (%s)", handle->m_lasterr);
+		return SCAP_FAILURE;
 	}
 
 	snprintf(filename, sizeof(filename), "%sraw", netroot);
 	if(scap_fd_read_ipv4_sockets_from_proc_fs(handle, filename, SCAP_L4_RAW, &sockets->sockets) == SCAP_FAILURE)
 	{
 		scap_fd_free_table(handle, &sockets->sockets);
-		return SCAP_FAILURE;		
+		snprintf(error, SCAP_LASTERR_SIZE, "Could not read ipv4 raw sockets (%s)", handle->m_lasterr);
+		return SCAP_FAILURE;
 	}
 
 	snprintf(filename, sizeof(filename), "%sunix", netroot);
 	if(scap_fd_read_unix_sockets_from_proc_fs(handle, filename, &sockets->sockets) == SCAP_FAILURE)
 	{
 		scap_fd_free_table(handle, &sockets->sockets);
+		snprintf(error, SCAP_LASTERR_SIZE, "Could not read unix sockets (%s)", handle->m_lasterr);
 		return SCAP_FAILURE;
 	}
 
@@ -1603,6 +1691,7 @@ int32_t scap_fd_read_sockets(scap_t *handle, char* procdir, struct scap_ns_socke
 	if(scap_fd_read_netlink_sockets_from_proc_fs(handle, filename, &sockets->sockets) == SCAP_FAILURE)
 	{
 		scap_fd_free_table(handle, &sockets->sockets);
+		snprintf(error, SCAP_LASTERR_SIZE, "Could not read netlink sockets (%s)", handle->m_lasterr);
 		return SCAP_FAILURE;
 	}
 
@@ -1613,24 +1702,27 @@ int32_t scap_fd_read_sockets(scap_t *handle, char* procdir, struct scap_ns_socke
 		if(scap_fd_read_ipv6_sockets_from_proc_fs(handle, filename, SCAP_L4_TCP, &sockets->sockets) == SCAP_FAILURE)
 		{
 			scap_fd_free_table(handle, &sockets->sockets);
-			return SCAP_FAILURE;		
+			snprintf(error, SCAP_LASTERR_SIZE, "Could not read ipv6 tcp sockets (%s)", handle->m_lasterr);
+			return SCAP_FAILURE;
 		}
 
 		snprintf(filename, sizeof(filename), "%sudp6", netroot);
-		if(scap_fd_read_ipv6_sockets_from_proc_fs(handle, filename, SCAP_L4_TCP, &sockets->sockets) == SCAP_FAILURE)
+		if(scap_fd_read_ipv6_sockets_from_proc_fs(handle, filename, SCAP_L4_UDP, &sockets->sockets) == SCAP_FAILURE)
 		{
 			scap_fd_free_table(handle, &sockets->sockets);
-			return SCAP_FAILURE;		
+			snprintf(error, SCAP_LASTERR_SIZE, "Could not read ipv6 udp sockets (%s)", handle->m_lasterr);
+			return SCAP_FAILURE;
 		}
 
 		snprintf(filename, sizeof(filename), "%sraw6", netroot);
-		if(scap_fd_read_ipv6_sockets_from_proc_fs(handle, filename, SCAP_L4_TCP, &sockets->sockets) == SCAP_FAILURE)
+		if(scap_fd_read_ipv6_sockets_from_proc_fs(handle, filename, SCAP_L4_RAW, &sockets->sockets) == SCAP_FAILURE)
 		{
 			scap_fd_free_table(handle, &sockets->sockets);
-			return SCAP_FAILURE;		
+			snprintf(error, SCAP_LASTERR_SIZE, "Could not read ipv6 raw sockets (%s)", handle->m_lasterr);
+			return SCAP_FAILURE;
 		}
     }
-    
+
 	return SCAP_SUCCESS;
 }
 
@@ -1754,6 +1846,7 @@ int32_t scap_fd_scan_fd_dir(scap_t *handle, char *procdir, scap_threadinfo *tinf
 			res = scap_fd_allocate_fdinfo(handle, &fdi, fd, SCAP_FD_FIFO);
 			if(SCAP_FAILURE == res)
 			{
+				snprintf(error, SCAP_LASTERR_SIZE, "can't allocate scap fd handle for fifo fd %ld", fd);
 				break;
 			}
 			res = scap_fd_handle_pipe(handle, f_name, tinfo, fdi, error);
@@ -1765,6 +1858,7 @@ int32_t scap_fd_scan_fd_dir(scap_t *handle, char *procdir, scap_threadinfo *tinf
 			res = scap_fd_allocate_fdinfo(handle, &fdi, fd, SCAP_FD_FILE_V2);
 			if(SCAP_FAILURE == res)
 			{
+				snprintf(error, SCAP_LASTERR_SIZE, "can't allocate scap fd handle for file fd %ld", fd);
 				break;
 			}
 			fdi->ino = sb.st_ino;
@@ -1774,6 +1868,7 @@ int32_t scap_fd_scan_fd_dir(scap_t *handle, char *procdir, scap_threadinfo *tinf
 			res = scap_fd_allocate_fdinfo(handle, &fdi, fd, SCAP_FD_DIRECTORY);
 			if(SCAP_FAILURE == res)
 			{
+				snprintf(error, SCAP_LASTERR_SIZE, "can't allocate scap fd handle for dir fd %ld", fd);
 				break;
 			}
 			fdi->ino = sb.st_ino;
@@ -1783,6 +1878,7 @@ int32_t scap_fd_scan_fd_dir(scap_t *handle, char *procdir, scap_threadinfo *tinf
 			res = scap_fd_allocate_fdinfo(handle, &fdi, fd, SCAP_FD_UNKNOWN);
 			if(SCAP_FAILURE == res)
 			{
+				snprintf(error, SCAP_LASTERR_SIZE, "can't allocate scap fd handle for sock fd %ld", fd);
 				break;
 			}
 			res = scap_fd_handle_socket(handle, f_name, tinfo, fdi, procdir, net_ns, sockets_by_ns, error);
@@ -1793,12 +1889,13 @@ int32_t scap_fd_scan_fd_dir(scap_t *handle, char *procdir, scap_threadinfo *tinf
 				{
 					scap_fd_free_fdinfo(&fdi);
 				}
-			} 
+			}
 			break;
 		default:
 			res = scap_fd_allocate_fdinfo(handle, &fdi, fd, SCAP_FD_UNSUPPORTED);
 			if(SCAP_FAILURE == res)
 			{
+				snprintf(error, SCAP_LASTERR_SIZE, "can't allocate scap fd handle for unsupported fd %ld", fd);
 				break;
 			}
 			fdi->ino = sb.st_ino;
@@ -1831,12 +1928,12 @@ int32_t scap_fd_scan_fd_dir(scap_t *handle, char *procdir, scap_threadinfo *tinf
 //
 // Internal helper function to output the fd table of a process
 //
-void scap_fd_print_table(scap_threadinfo *tinfo)
+void scap_fd_print_table(scap_t *handle, scap_threadinfo *tinfo)
 {
-	scap_fd_print_fd_table(tinfo->fdlist);
+	scap_fd_print_fd_table(handle, tinfo->fdlist);
 }
 
-void scap_fd_print_fd_table(scap_fdinfo *fds)
+void scap_fd_print_fd_table(scap_t *handle, scap_fdinfo *fds)
 {
 	scap_fdinfo *fdi;
 	scap_fdinfo *tfdi;
@@ -1844,7 +1941,7 @@ void scap_fd_print_fd_table(scap_fdinfo *fds)
 
 	HASH_ITER(hh, fds, fdi, tfdi)
 	{
-		if(scap_fd_info_to_string(fdi, str, SCAP_MAX_PATH_SIZE) != SCAP_SUCCESS)
+		if(scap_fd_info_to_string(handle, fdi, str, SCAP_MAX_PATH_SIZE) != SCAP_SUCCESS)
 		{
 			ASSERT(false);
 			snprintf(str, SCAP_MAX_PATH_SIZE, "N.A.");
