@@ -1,19 +1,20 @@
 /*
-Copyright (C) 2013-2014 Draios inc.
+Copyright (C) 2013-2018 Draios Inc dba Sysdig.
 
 This file is part of sysdig.
 
-sysdig is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-sysdig is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 
-You should have received a copy of the GNU General Public License
-along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #pragma once
@@ -325,6 +326,14 @@ public:
 	scap_dump_flags get_dump_flags(OUT bool* should_drop);
 #endif
 
+	/*!
+	  \brief Return whether or not falco should consider this
+	  event. (Generally, these events are automatically filtered
+	  out, but some events related to internal tracking are returned by next() anyway).
+	*/
+
+	bool falco_consider();
+
 // Doxygen doesn't understand VISIBILITY_PRIVATE
 #ifdef _DOXYGEN
 private:
@@ -366,9 +375,14 @@ private:
 		uint32_t nparams;
 		sinsp_evt_param par;
 
-		nparams = m_info->nparams;
+		// If we're reading a capture created with a newer version, it may contain
+		// new parameters. If instead we're reading an older version, the current
+		// event table entry may contain new parameters.
+		// Use the minimum between the two values.
+		nparams = m_info->nparams < m_pevt->nparams ? m_info->nparams : m_pevt->nparams;
 		uint16_t *lens = (uint16_t *)((char *)m_pevt + sizeof(struct ppm_evt_hdr));
-		char *valptr = (char *)lens + nparams * sizeof(uint16_t);
+		// The offset in the block is instead always based on the capture value.
+		char *valptr = (char *)lens + m_pevt->nparams * sizeof(uint16_t);
 		m_params.clear();
 
 		for(j = 0; j < nparams; j++)
