@@ -1,19 +1,20 @@
 /*
-Copyright (C) 2013-2014 Draios inc.
+Copyright (C) 2013-2018 Draios Inc dba Sysdig.
 
 This file is part of sysdig.
 
-sysdig is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-sysdig is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 
-You should have received a copy of the GNU General Public License
-along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 //
@@ -31,6 +32,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #endif
 
 #include <regex>
+#include <algorithm>
 
 #include "sinsp.h"
 #include "sinsp_int.h"
@@ -78,10 +80,8 @@ sinsp_filter_check_list::sinsp_filter_check_list()
 	add_filter_check(new sinsp_filter_check_container());
 	add_filter_check(new sinsp_filter_check_utils());
 	add_filter_check(new sinsp_filter_check_fdlist());
-#ifndef HAS_ANALYZER
-	add_filter_check(new sinsp_filter_check_k8s());
-#endif // HAS_ANALYZER
 #ifndef CYGWING_AGENT
+	add_filter_check(new sinsp_filter_check_k8s());
 	add_filter_check(new sinsp_filter_check_mesos());
 #endif
 	add_filter_check(new sinsp_filter_check_tracer());
@@ -193,6 +193,9 @@ bool flt_compare_uint64(cmpop op, uint64_t operand1, uint64_t operand2)
 	case CO_STARTSWITH:
 		throw sinsp_exception("'startswith' not supported for numeric filters");
 		return false;
+	case CO_ENDSWITH:
+		throw sinsp_exception("'endswith' not supported for numeric filters");
+		return false;
 	case CO_GLOB:
 		throw sinsp_exception("'glob' not supported for numeric filters");
 		return false;
@@ -227,6 +230,9 @@ bool flt_compare_int64(cmpop op, int64_t operand1, int64_t operand2)
 	case CO_STARTSWITH:
 		throw sinsp_exception("'startswith' not supported for numeric filters");
 		return false;
+        case CO_ENDSWITH:
+                throw sinsp_exception("'endswith' not supported for numeric filters");
+                return false;
 	case CO_GLOB:
 		throw sinsp_exception("'glob' not supported for numeric filters");
 		return false;
@@ -254,6 +260,8 @@ bool flt_compare_string(cmpop op, char* operand1, char* operand2)
 #endif
 	case CO_STARTSWITH:
 		return (strncmp(operand1, operand2, strlen(operand2)) == 0);
+	case CO_ENDSWITH: 
+		return (sinsp_utils::endswith(operand1, operand2));
 	case CO_GLOB:
 		return sinsp_utils::glob_match(operand2, operand1);
 	case CO_LT:
@@ -285,6 +293,8 @@ bool flt_compare_buffer(cmpop op, char* operand1, char* operand2, uint32_t op1_l
 		throw sinsp_exception("'icontains' not supported for buffer filters");
 	case CO_STARTSWITH:
 		return (memcmp(operand1, operand2, op2_len) == 0);
+	case CO_ENDSWITH: 
+		return (sinsp_utils::endswith(operand1, operand2, op1_len, op2_len));
 	case CO_GLOB:
 		throw sinsp_exception("'glob' not supported for buffer filters");
 	case CO_LT:
@@ -327,6 +337,9 @@ bool flt_compare_double(cmpop op, double operand1, double operand2)
 	case CO_STARTSWITH:
 		throw sinsp_exception("'startswith' not supported for numeric filters");
 		return false;
+	case CO_ENDSWITH:
+		throw sinsp_exception("'endswith' not supported for numeric filters");
+		return false;
 	case CO_GLOB:
 		throw sinsp_exception("'glob' not supported for numeric filters");
 		return false;
@@ -341,6 +354,7 @@ bool flt_compare_ipv4net(cmpop op, uint64_t operand1, ipv4net* operand2)
 	switch(op)
 	{
 	case CO_EQ:
+	case CO_IN:
 	{
 		return ((operand1 & operand2->m_netmask) == (operand2->m_ip & operand2->m_netmask));
 	}
@@ -355,6 +369,9 @@ bool flt_compare_ipv4net(cmpop op, uint64_t operand1, ipv4net* operand2)
 	case CO_STARTSWITH:
 		throw sinsp_exception("'startswith' not supported for numeric filters");
 		return false;
+	case CO_ENDSWITH:
+		throw sinsp_exception("'endswith' not supported for numeric filters");
+		return false;
 	case CO_GLOB:
 		throw sinsp_exception("'glob' not supported for numeric filters");
 		return false;
@@ -363,6 +380,58 @@ bool flt_compare_ipv4net(cmpop op, uint64_t operand1, ipv4net* operand2)
 	}
 }
 
+bool flt_compare_ipv6addr(cmpop op, ipv6addr *operand1, ipv6addr *operand2)
+{
+	switch(op)
+	{
+	case CO_EQ:
+	case CO_IN:
+		return *operand1 == *operand2;
+	case CO_NE:
+		return *operand1 != *operand2;
+	case CO_CONTAINS:
+		throw sinsp_exception("'contains' not supported for ipv6 addresses");
+		return false;
+	case CO_ICONTAINS:
+		throw sinsp_exception("'icontains' not supported for ipv6 addresses");
+		return false;
+	case CO_STARTSWITH:
+		throw sinsp_exception("'startswith' not supported for ipv6 addresses");
+		return false;
+	case CO_GLOB:
+		throw sinsp_exception("'glob' not supported for ipv6 addresses");
+		return false;
+	default:
+		throw sinsp_exception("comparison operator not supported for ipv6 addresses");
+	}
+}
+
+bool flt_compare_ipv6net(cmpop op, ipv6addr *operand1, ipv6addr *operand2)
+{
+	switch(op)
+	{
+	case CO_EQ:
+	case CO_IN:
+		return operand1->in_subnet(*operand2);
+	case CO_NE:
+		return !operand1->in_subnet(*operand2);
+	case CO_CONTAINS:
+		throw sinsp_exception("'contains' not supported for ipv6 networks");
+		return false;
+	case CO_ICONTAINS:
+		throw sinsp_exception("'icontains' not supported for ipv6 networks");
+		return false;
+	case CO_STARTSWITH:
+		throw sinsp_exception("'startswith' not supported for ipv6 networks");
+		return false;
+	case CO_GLOB:
+		throw sinsp_exception("'glob' not supported for ipv6 networks");
+		return false;
+	default:
+		throw sinsp_exception("comparison operator not supported for ipv6 networks");
+	}
+}
+
 bool flt_compare(cmpop op, ppm_param_type type, void* operand1, void* operand2, uint32_t op1_len, uint32_t op2_len)
 {
 	//
@@ -403,6 +472,36 @@ bool flt_compare(cmpop op, ppm_param_type type, void* operand1, void* operand2,
 		return flt_compare_uint64(op, (uint64_t)*(uint32_t*)operand1, (uint64_t)*(uint32_t*)operand2);
 	case PT_IPV4NET:
 		return flt_compare_ipv4net(op, (uint64_t)*(uint32_t*)operand1, (ipv4net*)operand2);
+	case PT_IPV6ADDR:
+		return flt_compare_ipv6addr(op, (ipv6addr *)operand1, (ipv6addr *)operand2);
+	case PT_IPV6NET:
+		return flt_compare_ipv6net(op, (ipv6addr *)operand1, (ipv6addr*)operand2);
+	case PT_IPADDR:
+		if(op1_len == sizeof(struct in_addr))
+		{
+			return flt_compare(op, PT_IPV4ADDR, operand1, operand2, op1_len, op2_len);
+		}
+		else if(op1_len == sizeof(struct in6_addr))
+		{
+			return flt_compare(op, PT_IPV6ADDR, operand1, operand2, op1_len, op2_len);
+		}
+		else
+		{
+			throw sinsp_exception("rawval_to_string called with IP address of incorrect size " + to_string(op1_len));
+		}
+	case PT_IPNET:
+		if(op1_len == sizeof(struct in_addr))
+		{
+			return flt_compare(op, PT_IPV4NET, operand1, operand2, op1_len, op2_len);
+		}
+		else if(op1_len == sizeof(struct in6_addr))
+		{
+			return flt_compare(op, PT_IPV6NET, operand1, operand2, op1_len, op2_len);
+		}
+		else
+		{
+			throw sinsp_exception("rawval_to_string called with IP network of incorrect size " + to_string(op1_len));
+		}
 	case PT_UINT64:
 	case PT_RELTIME:
 	case PT_ABSTIME:
@@ -501,6 +600,8 @@ bool flt_compare_avg(cmpop op,
 	case PT_FLAGS32:
 	case PT_BOOL:
 	case PT_IPV4ADDR:
+	case PT_IPV6ADDR:
+		// What does an average mean for ip addresses anyway?
 		u641 = ((uint64_t)*(uint32_t*)operand1) / cnt1;
 		u642 = ((uint64_t)*(uint32_t*)operand2) / cnt2;
 		ASSERT(cnt1 != 0 || u641 == 0);
@@ -550,23 +651,25 @@ void sinsp_filter_check::set_inspector(sinsp* inspector)
 	m_inspector = inspector;
 }
 
-Json::Value sinsp_filter_check::rawval_to_json(uint8_t* rawval, const filtercheck_field_info* finfo, uint32_t len)
+Json::Value sinsp_filter_check::rawval_to_json(uint8_t* rawval,
+					       ppm_param_type ptype,
+					       ppm_print_format print_format,
+					       uint32_t len)
 {
 	ASSERT(rawval != NULL);
-	ASSERT(finfo != NULL);
 
-	switch(finfo->m_type)
+	switch(ptype)
 	{
 		case PT_INT8:
-			if(finfo->m_print_format == PF_DEC ||
-			   finfo->m_print_format == PF_ID)
+			if(print_format == PF_DEC ||
+			   print_format == PF_ID)
 			{
 				return *(int8_t *)rawval;
 			}
-			else if(finfo->m_print_format == PF_OCT ||
-				finfo->m_print_format == PF_HEX)
+			else if(print_format == PF_OCT ||
+				print_format == PF_HEX)
 			{
-				return rawval_to_string(rawval, finfo, len);
+				return rawval_to_string(rawval, ptype, print_format, len);
 			}
 			else
 			{
@@ -575,15 +678,15 @@ Json::Value sinsp_filter_check::rawval_to_json(uint8_t* rawval, const filterchec
 			}
 
 		case PT_INT16:
-			if(finfo->m_print_format == PF_DEC ||
-			   finfo->m_print_format == PF_ID)
+			if(print_format == PF_DEC ||
+			   print_format == PF_ID)
 			{
 				return *(int16_t *)rawval;
 			}
-			else if(finfo->m_print_format == PF_OCT ||
-				finfo->m_print_format == PF_HEX)
+			else if(print_format == PF_OCT ||
+				print_format == PF_HEX)
 			{
-				return rawval_to_string(rawval, finfo, len);
+				return rawval_to_string(rawval, ptype, print_format, len);
 			}
 			else
 			{
@@ -592,15 +695,15 @@ Json::Value sinsp_filter_check::rawval_to_json(uint8_t* rawval, const filterchec
 			}
 
 		case PT_INT32:
-			if(finfo->m_print_format == PF_DEC ||
-			   finfo->m_print_format == PF_ID)
+			if(print_format == PF_DEC ||
+			   print_format == PF_ID)
 			{
 				return *(int32_t *)rawval;
 			}
-			else if(finfo->m_print_format == PF_OCT ||
-				finfo->m_print_format == PF_HEX)
+			else if(print_format == PF_OCT ||
+				print_format == PF_HEX)
 			{
-				return rawval_to_string(rawval, finfo, len);
+				return rawval_to_string(rawval, ptype, print_format, len);
 			}
 			else
 			{
@@ -610,27 +713,27 @@ Json::Value sinsp_filter_check::rawval_to_json(uint8_t* rawval, const filterchec
 
 		case PT_INT64:
 		case PT_PID:
-			if(finfo->m_print_format == PF_DEC ||
-			   finfo->m_print_format == PF_ID)
+			if(print_format == PF_DEC ||
+			   print_format == PF_ID)
 			{
 		 		return (Json::Value::Int64)*(int64_t *)rawval;
 			}
 			else
 			{
-				return rawval_to_string(rawval, finfo, len);
+				return rawval_to_string(rawval, ptype, print_format, len);
 			}
 
 		case PT_L4PROTO: // This can be resolved in the future
 		case PT_UINT8:
-			if(finfo->m_print_format == PF_DEC ||
-			   finfo->m_print_format == PF_ID)
+			if(print_format == PF_DEC ||
+			   print_format == PF_ID)
 			{
 				return *(uint8_t *)rawval;
 			}
-			else if(finfo->m_print_format == PF_OCT ||
-				finfo->m_print_format == PF_HEX)
+			else if(print_format == PF_OCT ||
+				print_format == PF_HEX)
 			{
-				return rawval_to_string(rawval, finfo, len);
+				return rawval_to_string(rawval, ptype, print_format, len);
 			}
 			else
 			{
@@ -640,15 +743,15 @@ Json::Value sinsp_filter_check::rawval_to_json(uint8_t* rawval, const filterchec
 
 		case PT_PORT: // This can be resolved in the future
 		case PT_UINT16:
-			if(finfo->m_print_format == PF_DEC ||
-			   finfo->m_print_format == PF_ID)
+			if(print_format == PF_DEC ||
+			   print_format == PF_ID)
 			{
 				return *(uint16_t *)rawval;
 			}
-			else if(finfo->m_print_format == PF_OCT ||
-				finfo->m_print_format == PF_HEX)
+			else if(print_format == PF_OCT ||
+				print_format == PF_HEX)
 			{
-				return rawval_to_string(rawval, finfo, len);
+				return rawval_to_string(rawval, ptype, print_format, len);
 			}
 			else
 			{
@@ -657,15 +760,15 @@ Json::Value sinsp_filter_check::rawval_to_json(uint8_t* rawval, const filterchec
 			}
 
 		case PT_UINT32:
-			if(finfo->m_print_format == PF_DEC ||
-			   finfo->m_print_format == PF_ID)
+			if(print_format == PF_DEC ||
+			   print_format == PF_ID)
 			{
 				return *(uint32_t *)rawval;
 			}
-			else if(finfo->m_print_format == PF_OCT ||
-				finfo->m_print_format == PF_HEX)
+			else if(print_format == PF_OCT ||
+				print_format == PF_HEX)
 			{
-				return rawval_to_string(rawval, finfo, len);
+				return rawval_to_string(rawval, ptype, print_format, len);
 			}
 			else
 			{
@@ -676,17 +779,17 @@ Json::Value sinsp_filter_check::rawval_to_json(uint8_t* rawval, const filterchec
 		case PT_UINT64:
 		case PT_RELTIME:
 		case PT_ABSTIME:
-			if(finfo->m_print_format == PF_DEC ||
-			   finfo->m_print_format == PF_ID)
+			if(print_format == PF_DEC ||
+			   print_format == PF_ID)
 			{
 				return (Json::Value::UInt64)*(uint64_t *)rawval;
 			}
 			else if(
-				finfo->m_print_format == PF_10_PADDED_DEC ||
-				finfo->m_print_format == PF_OCT ||
-				finfo->m_print_format == PF_HEX)
+				print_format == PF_10_PADDED_DEC ||
+				print_format == PF_OCT ||
+				print_format == PF_HEX)
 			{
-				return rawval_to_string(rawval, finfo, len);
+				return rawval_to_string(rawval, ptype, print_format, len);
 			}
 			else
 			{
@@ -706,34 +809,37 @@ Json::Value sinsp_filter_check::rawval_to_json(uint8_t* rawval, const filterchec
 		case PT_FSPATH:
 		case PT_BYTEBUF:
 		case PT_IPV4ADDR:
-			return rawval_to_string(rawval, finfo, len);
-
+		case PT_IPV6ADDR:
+	        case PT_IPADDR:
+			return rawval_to_string(rawval, ptype, print_format, len);
 		default:
 			ASSERT(false);
-			throw sinsp_exception("wrong event type " + to_string((long long) finfo->m_type));
+			throw sinsp_exception("wrong event type " + to_string((long long) ptype));
 	}
 }
 
-char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_field_info* finfo, uint32_t len)
+char* sinsp_filter_check::rawval_to_string(uint8_t* rawval,
+					   ppm_param_type ptype,
+					   ppm_print_format print_format,
+					   uint32_t len)
 {
 	char* prfmt;
 
 	ASSERT(rawval != NULL);
-	ASSERT(finfo != NULL);
 
-	switch(finfo->m_type)
+	switch(ptype)
 	{
 		case PT_INT8:
-			if(finfo->m_print_format == PF_OCT)
+			if(print_format == PF_OCT)
 			{
 				prfmt = (char*)"%" PRIo8;
 			}
-			else if(finfo->m_print_format == PF_DEC ||
-				finfo->m_print_format == PF_ID)
+			else if(print_format == PF_DEC ||
+				print_format == PF_ID)
 			{
 				prfmt = (char*)"%" PRId8;
 			}
-			else if(finfo->m_print_format == PF_HEX)
+			else if(print_format == PF_HEX)
 			{
 				prfmt = (char*)"%" PRIX8;
 			}
@@ -748,16 +854,16 @@ char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_fi
 					 prfmt, *(int8_t *)rawval);
 			return m_getpropertystr_storage;
 		case PT_INT16:
-			if(finfo->m_print_format == PF_OCT)
+			if(print_format == PF_OCT)
 			{
 				prfmt = (char*)"%" PRIo16;
 			}
-			else if(finfo->m_print_format == PF_DEC ||
-				finfo->m_print_format == PF_ID)
+			else if(print_format == PF_DEC ||
+				print_format == PF_ID)
 			{
 				prfmt = (char*)"%" PRId16;
 			}
-			else if(finfo->m_print_format == PF_HEX)
+			else if(print_format == PF_HEX)
 			{
 				prfmt = (char*)"%" PRIX16;
 			}
@@ -772,16 +878,16 @@ char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_fi
 					 prfmt, *(int16_t *)rawval);
 			return m_getpropertystr_storage;
 		case PT_INT32:
-			if(finfo->m_print_format == PF_OCT)
+			if(print_format == PF_OCT)
 			{
 				prfmt = (char*)"%" PRIo32;
 			}
-			else if(finfo->m_print_format == PF_DEC ||
-				finfo->m_print_format == PF_ID)
+			else if(print_format == PF_DEC ||
+				print_format == PF_ID)
 			{
 				prfmt = (char*)"%" PRId32;
 			}
-			else if(finfo->m_print_format == PF_HEX)
+			else if(print_format == PF_HEX)
 			{
 				prfmt = (char*)"%" PRIX32;
 			}
@@ -798,20 +904,20 @@ char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_fi
 		case PT_INT64:
 		case PT_PID:
 		case PT_ERRNO:
-			if(finfo->m_print_format == PF_OCT)
+			if(print_format == PF_OCT)
 			{
 				prfmt = (char*)"%" PRIo64;
 			}
-			else if(finfo->m_print_format == PF_DEC ||
-				finfo->m_print_format == PF_ID)
+			else if(print_format == PF_DEC ||
+				print_format == PF_ID)
 			{
 				prfmt = (char*)"%" PRId64;
 			}
-			else if(finfo->m_print_format == PF_10_PADDED_DEC)
+			else if(print_format == PF_10_PADDED_DEC)
 			{
 				prfmt = (char*)"%09" PRId64;
 			}
-			else if(finfo->m_print_format == PF_HEX)
+			else if(print_format == PF_HEX)
 			{
 				prfmt = (char*)"%" PRIX64;
 			}
@@ -826,16 +932,16 @@ char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_fi
 			return m_getpropertystr_storage;
 		case PT_L4PROTO: // This can be resolved in the future
 		case PT_UINT8:
-			if(finfo->m_print_format == PF_OCT)
+			if(print_format == PF_OCT)
 			{
 				prfmt = (char*)"%" PRIo8;
 			}
-			else if(finfo->m_print_format == PF_DEC ||
-				finfo->m_print_format == PF_ID)
+			else if(print_format == PF_DEC ||
+				print_format == PF_ID)
 			{
 				prfmt = (char*)"%" PRIu8;
 			}
-			else if(finfo->m_print_format == PF_HEX)
+			else if(print_format == PF_HEX)
 			{
 				prfmt = (char*)"%" PRIu8;
 			}
@@ -851,16 +957,16 @@ char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_fi
 			return m_getpropertystr_storage;
 		case PT_PORT: // This can be resolved in the future
 		case PT_UINT16:
-			if(finfo->m_print_format == PF_OCT)
+			if(print_format == PF_OCT)
 			{
 				prfmt = (char*)"%" PRIo16;
 			}
-			else if(finfo->m_print_format == PF_DEC ||
-				finfo->m_print_format == PF_ID)
+			else if(print_format == PF_DEC ||
+				print_format == PF_ID)
 			{
 				prfmt = (char*)"%" PRIu16;
 			}
-			else if(finfo->m_print_format == PF_HEX)
+			else if(print_format == PF_HEX)
 			{
 				prfmt = (char*)"%" PRIu16;
 			}
@@ -875,16 +981,16 @@ char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_fi
 					 prfmt, *(uint16_t *)rawval);
 			return m_getpropertystr_storage;
 		case PT_UINT32:
-			if(finfo->m_print_format == PF_OCT)
+			if(print_format == PF_OCT)
 			{
 				prfmt = (char*)"%" PRIo32;
 			}
-			else if(finfo->m_print_format == PF_DEC ||
-				finfo->m_print_format == PF_ID)
+			else if(print_format == PF_DEC ||
+				print_format == PF_ID)
 			{
 				prfmt = (char*)"%" PRIu32;
 			}
-			else if(finfo->m_print_format == PF_HEX)
+			else if(print_format == PF_HEX)
 			{
 				prfmt = (char*)"%" PRIu32;
 			}
@@ -901,20 +1007,20 @@ char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_fi
 		case PT_UINT64:
 		case PT_RELTIME:
 		case PT_ABSTIME:
-			if(finfo->m_print_format == PF_OCT)
+			if(print_format == PF_OCT)
 			{
 				prfmt = (char*)"%" PRIo64;
 			}
-			else if(finfo->m_print_format == PF_DEC ||
-				finfo->m_print_format == PF_ID)
+			else if(print_format == PF_DEC ||
+				print_format == PF_ID)
 			{
 				prfmt = (char*)"%" PRIu64;
 			}
-			else if(finfo->m_print_format == PF_10_PADDED_DEC)
+			else if(print_format == PF_10_PADDED_DEC)
 			{
 				prfmt = (char*)"%09" PRIu64;
 			}
-			else if(finfo->m_print_format == PF_HEX)
+			else if(print_format == PF_HEX)
 			{
 				prfmt = (char*)"%" PRIX64;
 			}
@@ -973,6 +1079,35 @@ char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_fi
 						rawval[2],
 						rawval[3]);
 			return m_getpropertystr_storage;
+		case PT_IPV6ADDR:
+		{
+			char address[100];
+
+			if(NULL == inet_ntop(AF_INET6, rawval, address, 100))
+			{
+				strcpy(address, "<NA>");
+			}
+
+			strncpy(m_getpropertystr_storage,
+				address,
+				100);
+
+			return m_getpropertystr_storage;
+		}
+	        case PT_IPADDR:
+			if(len == sizeof(struct in_addr))
+			{
+				return rawval_to_string(rawval, PT_IPV4ADDR, print_format, len);
+			}
+			else if(len == sizeof(struct in6_addr))
+			{
+				return rawval_to_string(rawval, PT_IPV6ADDR, print_format, len);
+			}
+			else
+			{
+				throw sinsp_exception("rawval_to_string called with IP address of incorrect size " + to_string(len));
+			}
+
 		case PT_DOUBLE:
 			snprintf(m_getpropertystr_storage,
 					 sizeof(m_getpropertystr_storage),
@@ -980,7 +1115,7 @@ char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_fi
 			return m_getpropertystr_storage;
 		default:
 			ASSERT(false);
-			throw sinsp_exception("wrong event type " + to_string((long long) finfo->m_type));
+			throw sinsp_exception("wrong event type " + to_string((long long) ptype));
 	}
 }
 
@@ -994,7 +1129,7 @@ char* sinsp_filter_check::tostring(sinsp_evt* evt)
 		return NULL;
 	}
 
-	return rawval_to_string(rawval, m_field, len);
+	return rawval_to_string(rawval, m_field->m_type, m_field->m_print_format, len);
 }
 
 Json::Value sinsp_filter_check::tojson(sinsp_evt* evt)
@@ -1009,7 +1144,7 @@ Json::Value sinsp_filter_check::tojson(sinsp_evt* evt)
 		{
 			return Json::nullValue;
 		}
-		return rawval_to_json(rawval, m_field, len);
+		return rawval_to_json(rawval, m_field->m_type, m_field->m_print_format, len);
 	}
 
 	return jsonval;
@@ -1133,32 +1268,59 @@ bool sinsp_filter_check::flt_compare(cmpop op, ppm_param_type type, void* operan
 {
 	if (op == CO_IN || op == CO_PMATCH)
 	{
-		// For raw strings, the length may not be set. So we do a strlen to find it.
-		if(type == PT_CHARBUF && op1_len == 0)
+		// Certain filterchecks can't be done as a set
+		// membership test/group match. For these, just loop over the
+		// values and see if any value is equal.
+		switch(type)
 		{
-			op1_len = strlen((char *) operand1);
-		}
-
-		filter_value_t item((uint8_t *) operand1, op1_len);
-
-		if (op == CO_IN)
-		{
-			if(op1_len >= m_val_storages_min_size &&
-			   op1_len <= m_val_storages_max_size &&
-			   m_val_storages_members.find(item) != m_val_storages_members.end())
+		case PT_IPV4NET:
+		case PT_IPV6NET:
+		case PT_SOCKADDR:
+		case PT_SOCKTUPLE:
+		case PT_FDLIST:
+		case PT_FSPATH:
+		case PT_SIGSET:
+			for (uint16_t i=0; i < m_val_storages.size(); i++)
 			{
-				return true;
+				if (::flt_compare(CO_EQ,
+						  type,
+						  operand1,
+						  filter_value_p(i)))
+				{
+					return true;
+				}
 			}
-		}
-		else
-		{
-			if (m_val_storages_paths.match(item))
+			return false;
+			break;
+		default:
+			// For raw strings, the length may not be set. So we do a strlen to find it.
+			if(type == PT_CHARBUF && op1_len == 0)
 			{
-				return true;
+				op1_len = strlen((char *) operand1);
 			}
-		}
 
-		return false;
+			filter_value_t item((uint8_t *) operand1, op1_len);
+
+			if (op == CO_IN)
+			{
+				if(op1_len >= m_val_storages_min_size &&
+				   op1_len <= m_val_storages_max_size &&
+				   m_val_storages_members.find(item) != m_val_storages_members.end())
+				{
+					return true;
+				}
+			}
+			else
+			{
+				if (m_val_storages_paths.match(item))
+				{
+					return true;
+				}
+			}
+
+			return false;
+			break;
+		}
 	}
 	else
 	{
@@ -1671,6 +1833,11 @@ cmpop sinsp_filter_compiler::next_comparison_operator()
 		m_scanpos += 10;
 		return CO_STARTSWITH;
 	}
+	else if(compare_no_consume("endswith"))
+	{
+		m_scanpos += 8;
+		return CO_ENDSWITH;
+	}
 	else if(compare_no_consume("glob"))
 	{
 		m_scanpos += 4;
@@ -2041,10 +2208,30 @@ sinsp_filter* sinsp_filter_compiler::compile_()
 
 sinsp_evttype_filter::sinsp_evttype_filter()
 {
-	memset(m_filter_by_evttype, 0, PPM_EVENT_MAX * sizeof(list<sinsp_filter *> *));
 }
 
 sinsp_evttype_filter::~sinsp_evttype_filter()
+{
+	for(const auto &val : m_filters)
+	{
+		delete val.second->filter;
+		delete val.second;
+	}
+
+	for(auto &ruleset : m_rulesets)
+	{
+		delete ruleset;
+	}
+	m_filters.clear();
+}
+
+sinsp_evttype_filter::ruleset_filters::ruleset_filters()
+{
+	memset(m_filter_by_evttype, 0, PPM_EVENT_MAX * sizeof(list<filter_wrapper *> *));
+	memset(m_filter_by_syscall, 0, PPM_SC_MAX * sizeof(list<filter_wrapper *> *));
+}
+
+sinsp_evttype_filter::ruleset_filters::~ruleset_filters()
 {
 	for(int i = 0; i < PPM_EVENT_MAX; i++)
 	{
@@ -2055,60 +2242,178 @@ sinsp_evttype_filter::~sinsp_evttype_filter()
 		}
 	}
 
-	m_catchall_evttype_filters.clear();
-
-	for(const auto &val : m_evttype_filters)
+	for(int i = 0; i < PPM_SC_MAX; i++)
 	{
-		delete val.second->filter;
-		delete val.second;
+		if(m_filter_by_syscall[i])
+		{
+			delete m_filter_by_syscall[i];
+			m_filter_by_syscall[i] = NULL;
+		}
 	}
-	m_evttype_filters.clear();
 }
 
-sinsp_evttype_filter::filter_wrapper::filter_wrapper()
-	: enabled{true}
+void sinsp_evttype_filter::ruleset_filters::add_filter(filter_wrapper *wrap)
 {
+	for(uint32_t etype = 0; etype < PPM_EVENT_MAX; etype++)
+	{
+		if(wrap->evttypes[etype])
+		{
+			if(!m_filter_by_evttype[etype])
+			{
+				m_filter_by_evttype[etype] = new std::list<filter_wrapper *>();
+			}
+
+			m_filter_by_evttype[etype]->push_back(wrap);
+		}
+	}
+
+	for(uint32_t syscall = 0; syscall < PPM_SC_MAX; syscall++)
+	{
+		if(wrap->syscalls[syscall])
+		{
+			if(!m_filter_by_syscall[syscall])
+			{
+				m_filter_by_syscall[syscall] = new std::list<filter_wrapper *>();
+			}
+
+			m_filter_by_syscall[syscall]->push_back(wrap);
+		}
+	}
 }
 
-sinsp_evttype_filter::filter_wrapper::~filter_wrapper()
+void sinsp_evttype_filter::ruleset_filters::remove_filter(filter_wrapper *wrap)
 {
+	for(uint32_t etype = 0; etype < PPM_EVENT_MAX; etype++)
+	{
+		if(wrap->evttypes[etype])
+		{
+			if(m_filter_by_evttype[etype])
+			{
+				m_filter_by_evttype[etype]->erase(std::remove(m_filter_by_evttype[etype]->begin(),
+									      m_filter_by_evttype[etype]->end(),
+									      wrap),
+								  m_filter_by_evttype[etype]->end());
+
+				if(m_filter_by_evttype[etype]->size() == 0)
+				{
+					delete m_filter_by_evttype[etype];
+					m_filter_by_evttype[etype] = NULL;
+				}
+			}
+		}
+	}
+
+	for(uint32_t syscall = 0; syscall < PPM_SC_MAX; syscall++)
+	{
+		if(wrap->syscalls[syscall])
+		{
+			if(m_filter_by_syscall[syscall])
+			{
+				m_filter_by_syscall[syscall]->erase(std::remove(m_filter_by_syscall[syscall]->begin(),
+										m_filter_by_syscall[syscall]->end(),
+										wrap),
+								    m_filter_by_syscall[syscall]->end());
+
+				if(m_filter_by_syscall[syscall]->size() == 0)
+				{
+					delete m_filter_by_syscall[syscall];
+					m_filter_by_syscall[syscall] = NULL;
+				}
+			}
+		}
+	}
 }
 
+
+bool sinsp_evttype_filter::ruleset_filters::run(sinsp_evt *evt)
+{
+	list<filter_wrapper *> *filters;
+
+ 	uint16_t etype = evt->m_pevt->type;
+
+	if(etype == PPME_GENERIC_E || etype == PPME_GENERIC_X)
+	{
+		sinsp_evt_param *parinfo = evt->get_param(0);
+		ASSERT(parinfo->m_len == sizeof(uint16_t));
+		uint16_t evid = *(uint16_t *)parinfo->m_val;
+
+		filters = m_filter_by_syscall[evid];
+	}
+	else
+	{
+		filters = m_filter_by_evttype[etype];
+	}
+
+	if (!filters) {
+		return false;
+	}
+
+	for (auto &wrap : *filters)
+	{
+		if(wrap->filter->run(evt))
+		{
+			return true;
+		}
+	}
+
+	return false;
+}
+
+void sinsp_evttype_filter::ruleset_filters::evttypes_for_ruleset(std::vector<bool> &evttypes)
+{
+	evttypes.assign(PPM_EVENT_MAX+1, false);
+
+	for(uint32_t etype = 0; etype < PPM_EVENT_MAX; etype++)
+	{
+		list<filter_wrapper *> *filters = m_filter_by_evttype[etype];
+		if(filters)
+		{
+			evttypes[etype] = true;
+		}
+	}
+}
+
+void sinsp_evttype_filter::ruleset_filters::syscalls_for_ruleset(std::vector<bool> &syscalls)
+{
+	syscalls.assign(PPM_SC_MAX+1, false);
+
+	for(uint32_t evid = 0; evid < PPM_SC_MAX; evid++)
+	{
+		list<filter_wrapper *> *filters = m_filter_by_syscall[evid];
+		if(filters)
+		{
+			syscalls[evid] = true;
+		}
+	}
+}
+
+
 void sinsp_evttype_filter::add(string &name,
 			       set<uint32_t> &evttypes,
+			       set<uint32_t> &syscalls,
 			       set<string> &tags,
 			       sinsp_filter *filter)
 {
 	filter_wrapper *wrap = new filter_wrapper();
 	wrap->filter = filter;
 
-	wrap->evttypes.assign(PPM_EVENT_MAX+1, false);
+	// If no evttypes or syscalls are specified, the filter is
+	// enabled for all evttypes/syscalls.
+	bool def = ((evttypes.size() == 0 && syscalls.size() == 0) ? true : false);
+
+	wrap->evttypes.assign(PPM_EVENT_MAX+1, def);
 	for(auto &evttype : evttypes)
 	{
 		wrap->evttypes[evttype] = true;
 	}
 
-	m_evttype_filters.insert(pair<string,filter_wrapper *>(name, wrap));
-
-	if(evttypes.size() == 0)
+	wrap->syscalls.assign(PPM_SC_MAX+1, def);
+	for(auto &syscall : syscalls)
 	{
-		m_catchall_evttype_filters.push_back(wrap);
+		wrap->syscalls[syscall] = true;
 	}
-	else
-	{
 
-		for(const auto &evttype: evttypes)
-		{
-			list<filter_wrapper *> *filters = m_filter_by_evttype[evttype];
-			if(filters == NULL)
-			{
-				filters = new list<filter_wrapper*>();
-				m_filter_by_evttype[evttype] = filters;
-			}
-
-			filters->push_back(wrap);
-		}
-	}
+	m_filters.insert(pair<string,filter_wrapper *>(name, wrap));
 
 	for(const auto &tag: tags)
 	{
@@ -2129,104 +2434,68 @@ void sinsp_evttype_filter::enable(const string &pattern, bool enabled, uint16_t
 {
 	regex re(pattern);
 
-	for(const auto &val : m_evttype_filters)
+	while (m_rulesets.size() < (size_t) ruleset + 1)
+	{
+		m_rulesets.push_back(new ruleset_filters());
+	}
+
+	for(const auto &val : m_filters)
 	{
 		if (regex_match(val.first, re))
 		{
-			if(val.second->enabled.size() < (size_t) (ruleset + 1))
+			if(enabled)
 			{
-				val.second->enabled.resize(ruleset + 1);
+				m_rulesets[ruleset]->add_filter(val.second);
+			}
+			else
+			{
+				m_rulesets[ruleset]->remove_filter(val.second);
 			}
-			val.second->enabled[ruleset] = enabled;
 		}
 	}
 }
 
 void sinsp_evttype_filter::enable_tags(const set<string> &tags, bool enabled, uint16_t ruleset)
 {
+	while (m_rulesets.size() < (size_t) ruleset + 1)
+	{
+		m_rulesets.push_back(new ruleset_filters());
+	}
+
 	for(const auto &tag : tags)
 	{
 		for(const auto &wrap : m_filter_by_tag[tag])
 		{
-			if(wrap->enabled.size() < (size_t) (ruleset + 1))
+			if(enabled)
 			{
-				wrap->enabled.resize(ruleset + 1);
+				m_rulesets[ruleset]->add_filter(wrap);
+			}
+			else
+			{
+				m_rulesets[ruleset]->remove_filter(wrap);
 			}
-			wrap->enabled[ruleset] = enabled;
 		}
 	}
 }
 
 bool sinsp_evttype_filter::run(sinsp_evt *evt, uint16_t ruleset)
 {
-	//
-	// First run any catchall event type filters (ones that did not
-	// explicitly specify any event type.
-	//
-	for(filter_wrapper *wrap : m_catchall_evttype_filters)
+	if(m_rulesets.size() < (size_t) ruleset + 1)
 	{
-		if(wrap->enabled.size() >= (size_t) (ruleset + 1) &&
-		   wrap->enabled[ruleset] &&
-		   wrap->filter->run(evt))
-		{
-			return true;
-		}
+		return false;
 	}
 
-        list<filter_wrapper *> *filters = m_filter_by_evttype[evt->m_pevt->type];
-
-	if(filters)
-	{
-		for(filter_wrapper *wrap : *filters)
-		{
-			if(wrap->enabled.size() >= (size_t) (ruleset + 1) &&
-			   wrap->enabled[ruleset] &&
-			   wrap->filter->run(evt))
-			{
-				return true;
-			}
-		}
-	}
-
-	return false;
-}
-
-
-// Solely used for code sharing in evttypes_for_rulset
-void sinsp_evttype_filter::check_filter_wrappers(std::vector<bool> &evttypes,
-						 uint32_t etype,
-						 std::list<filter_wrapper *> &filters,
-						 uint16_t ruleset)
-{
-	for(filter_wrapper *wrap : filters)
-	{
-		if(wrap->enabled.size() >= (size_t) (ruleset + 1) &&
-		   wrap->enabled[ruleset])
-		{
-			evttypes[etype] = true;
-			break;
-		}
-	}
+	return m_rulesets[ruleset]->run(evt);
 }
 
 void sinsp_evttype_filter::evttypes_for_ruleset(std::vector<bool> &evttypes, uint16_t ruleset)
 {
-	evttypes.assign(PPM_EVENT_MAX+1, false);
-
-	for(uint32_t etype = 0; etype < PPM_EVENT_MAX; etype++)
-	{
-		// Catchall filters (ones that don't explicitly refer
-		// to a type) must run for all event types.
-		check_filter_wrappers(evttypes, etype, m_catchall_evttype_filters, ruleset);
-
-		if(!evttypes[etype])
-		{
-			list<filter_wrapper *> *filters = m_filter_by_evttype[etype];
-			if(filters)
-			{
-				check_filter_wrappers(evttypes, etype, *filters, ruleset);
-			}
-		}
-	}
+	return m_rulesets[ruleset]->evttypes_for_ruleset(evttypes);
 }
+
+void sinsp_evttype_filter::syscalls_for_ruleset(std::vector<bool> &syscalls, uint16_t ruleset)
+{
+	return m_rulesets[ruleset]->syscalls_for_ruleset(syscalls);
+}
+
 #endif // HAS_FILTERING
