@@ -1,19 +1,20 @@
 /*
-Copyright (C) 2013-2014 Draios inc.
+Copyright (C) 2013-2018 Draios Inc dba Sysdig.
 
 This file is part of sysdig.
 
-sysdig is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-sysdig is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 
-You should have received a copy of the GNU General Public License
-along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #pragma once
@@ -32,6 +33,7 @@ class sinsp_filter_check_reference;
 bool flt_compare(cmpop op, ppm_param_type type, void* operand1, void* operand2, uint32_t op1_len = 0, uint32_t op2_len = 0);
 bool flt_compare_avg(cmpop op, ppm_param_type type, void* operand1, void* operand2, uint32_t op1_len, uint32_t op2_len, uint32_t cnt1, uint32_t cnt2);
 bool flt_compare_ipv4net(cmpop op, uint64_t operand1, ipv4net* operand2);
+bool flt_compare_ipv6net(cmpop op, ipv6addr *operand1, ipv6addr* operand2);
 
 char* flt_to_string(uint8_t* rawval, filtercheck_field_info* finfo);
 int32_t gmt2local(time_t t);
@@ -145,8 +147,11 @@ public:
 protected:
 	bool flt_compare(cmpop op, ppm_param_type type, void* operand1, uint32_t op1_len = 0, uint32_t op2_len = 0);
 
-	char* rawval_to_string(uint8_t* rawval, const filtercheck_field_info* finfo, uint32_t len);
-	Json::Value rawval_to_json(uint8_t* rawval, const filtercheck_field_info* finfo, uint32_t len);
+	char* rawval_to_string(uint8_t* rawval,
+			       ppm_param_type ptype,
+			       ppm_print_format print_format,
+			       uint32_t len);
+	Json::Value rawval_to_json(uint8_t* rawval, ppm_param_type ptype, ppm_print_format print_format, uint32_t len);
 	void string_to_rawval(const char* str, uint32_t len, ppm_param_type ptype);
 
 	char m_getpropertystr_storage[1024];
@@ -284,6 +289,10 @@ public:
 		TYPE_RNET = 31,
 		TYPE_IS_CONNECTED = 32,
 		TYPE_NAME_CHANGED = 33,
+		TYPE_CLIENTIP_NAME = 34,
+		TYPE_SERVERIP_NAME = 35,
+		TYPE_LIP_NAME = 36,
+		TYPE_RIP_NAME = 37
 	};
 
 	enum fd_type
@@ -308,6 +317,7 @@ public:
 	bool compare_ip(sinsp_evt *evt);
 	bool compare_net(sinsp_evt *evt);
 	bool compare_port(sinsp_evt *evt);
+	bool compare_domain(sinsp_evt *evt);
 	bool compare(sinsp_evt *evt);
 
 	sinsp_threadinfo* m_tinfo;
@@ -533,6 +543,8 @@ public:
 		TYPE_NAME = 1,
 		TYPE_HOMEDIR = 2,
 		TYPE_SHELL = 3,
+		TYPE_LOGINUID = 4,
+		TYPE_LOGINNAME = 5,
 	};
 
 	sinsp_filter_check_user();
@@ -755,7 +767,10 @@ public:
 		TYPE_CONTAINER_MOUNT_DEST,
 		TYPE_CONTAINER_MOUNT_MODE,
 		TYPE_CONTAINER_MOUNT_RDWR,
-		TYPE_CONTAINER_MOUNT_PROPAGATION
+		TYPE_CONTAINER_MOUNT_PROPAGATION,
+		TYPE_CONTAINER_IMAGE_REPOSITORY,
+		TYPE_CONTAINER_IMAGE_TAG,
+		TYPE_CONTAINER_IMAGE_DIGEST,
 	};
 
 	sinsp_filter_check_container();
@@ -858,7 +873,7 @@ private:
 	char m_addrbuff[100];
 };
 
-#ifndef HAS_ANALYZER
+#ifndef CYGWING_AGENT
 
 class sinsp_filter_check_k8s : public sinsp_filter_check
 {
@@ -911,7 +926,7 @@ private:
 	string m_tstr;
 };
 
-#endif // HAS_ANALYZER
+#endif // CYGWING_AGENT
 
 #ifndef CYGWING_AGENT
 class sinsp_filter_check_mesos : public sinsp_filter_check
