@@ -1,3 +1,21 @@
+/*
+Copyright (C) 2013-2018 Draios Inc dba Sysdig.
+
+This file is part of sysdig.
+
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+
+*/
 #include "sinsp.h"
 #include "sinsp_int.h"
 
@@ -53,6 +71,10 @@ cmpop string_to_cmpop(const char* str)
 	{
 		return CO_STARTSWITH;
 	}
+	else if(strcmp(str, "endswith") == 0)
+	{
+		return CO_ENDSWITH;
+	}
 	else if(strcmp(str, "in") == 0)
 	{
 		return CO_IN;
@@ -65,6 +87,10 @@ cmpop string_to_cmpop(const char* str)
 	{
 		return CO_EXISTS;
 	}
+	else if(strcmp(str, "glob") == 0)
+	{
+		return CO_GLOB;
+	}
 	else
 	{
 		throw sinsp_exception("filter error: invalid comparison operator: " + string(str));
