@@ -1,19 +1,20 @@
 /*
-Copyright (C) 2013-2014 Draios inc.
+Copyright (C) 2013-2018 Draios Inc dba Sysdig.
 
 This file is part of sysdig.
 
-sysdig is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-sysdig is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 
-You should have received a copy of the GNU General Public License
-along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /*!
@@ -391,6 +392,7 @@ public:
 
 	void add_evttype_filter(std::string &name,
 				std::set<uint32_t> &evttypes,
+				std::set<uint32_t> &syscalls,
 				std::set<std::string> &tags,
 				sinsp_filter* filter);
 
@@ -507,6 +509,7 @@ public:
 	   of failure.
 	*/
 	sinsp_threadinfo* get_thread(int64_t tid, bool query_os_if_not_found, bool lookup_only);
+	threadinfo_map_t::ptr_t get_thread_ref(int64_t tid, bool query_os_if_not_found, bool lookup_only);
 
 	/*!
 	  \brief Return the table with all the machine users.
@@ -520,6 +523,18 @@ public:
 	*/
 	const unordered_map<uint32_t, scap_userinfo*>* get_userlist();
 
+	/*!
+	  \brief Lookup for user in the user table.
+
+ 	  \return the \ref scap_userinfo object containing full user information,
+ 	   if user not found, returns NULL.
+
+ 	  \note this call works with file captures as well, because the user
+	   table is stored in the trace files. In that case, the returned
+	   user list is the one of the machine where the capture happened.
+	*/
+	scap_userinfo* get_user(uint32_t uid);
+
 	/*!
 	  \brief Return the table with all the machine user groups.
 
@@ -541,7 +556,7 @@ public:
 	void get_capture_stats(scap_stats* stats);
 
 	void set_max_thread_table_size(uint32_t value);
-	
+
 #ifdef GATHER_INTERNAL_STATS
 	sinsp_stats get_stats();
 #endif
@@ -820,6 +835,7 @@ public:
 	}
 	void set_simpledriver_mode();
 	vector<long> get_n_tracepoint_hit();
+	void set_bpf_probe(const string& bpf_probe);
 
 	static unsigned num_possible_cpus();
 #ifdef CYGWING_AGENT
@@ -828,8 +844,35 @@ public:
 		return scap_get_wmi_handle(m_h);
 	}
 #endif
+
+	static inline bool falco_consider_evtnum(uint16_t etype)
+	{
+		enum ppm_event_flags flags = g_infotables.m_event_info[etype].flags;
+
+		return ! (flags & sinsp::falco_skip_flags());
+	}
+
+	static inline bool falco_consider_syscallid(uint16_t scid)
+	{
+		enum ppm_event_flags flags = g_infotables.m_syscall_info_table[scid].flags;
+
+		return ! (flags & sinsp::falco_skip_flags());
+	}
+
+	// Add comm to the list of comms for which the inspector
+	// should not return events.
+	bool suppress_events_comm(const std::string &comm);
+
+	bool check_suppressed(int64_t tid);
+
+	void set_query_docker_image_info(bool query_image_info);
+
 VISIBILITY_PRIVATE
 
+        static inline ppm_event_flags falco_skip_flags()
+        {
+		return (ppm_event_flags) (EF_SKIPPARSERESET | EF_UNUSED | EF_DROP_FALCO);
+        }
 // Doxygen doesn't understand VISIBILITY_PRIVATE
 #ifdef _DOXYGEN
 private:
@@ -842,36 +885,40 @@ private:
 	void import_user_list();
 	void add_protodecoders();
 
-	void add_thread(const sinsp_threadinfo& ptinfo);
+	void add_thread(const sinsp_threadinfo* ptinfo);
 	void remove_thread(int64_t tid, bool force);
+
 	//
 	// Note: lookup_only should be used when the query for the thread is made
 	//       not as a consequence of an event for that thread arriving, but
 	//       just for lookup reason. In that case, m_lastaccess_ts is not updated
 	//       and m_last_tinfo is not set.
 	//
-	inline sinsp_threadinfo* find_thread(int64_t tid, bool lookup_only)
+	inline threadinfo_map_t::ptr_t find_thread(int64_t tid, bool lookup_only)
 	{
-		threadinfo_map_iterator_t it;
-
+		threadinfo_map_t::ptr_t thr;
 		//
 		// Try looking up in our simple cache
 		//
-		if(m_thread_manager->m_last_tinfo && tid == m_thread_manager->m_last_tid)
+		if(tid == m_thread_manager->m_last_tid)
 		{
+			thr = m_thread_manager->m_last_tinfo.lock();
+			if (thr)
+			{
 	#ifdef GATHER_INTERNAL_STATS
-			m_thread_manager->m_cached_lookups->increment();
+				m_thread_manager->m_cached_lookups->increment();
 	#endif
-			m_thread_manager->m_last_tinfo->m_lastaccess_ts = m_lastevent_ts;
-			return m_thread_manager->m_last_tinfo;
+				thr->m_lastaccess_ts = m_lastevent_ts;
+				return thr;
+			}
 		}
 
 		//
 		// Caching failed, do a real lookup
 		//
-		it = m_thread_manager->m_threadtable.find(tid);
+		thr = m_thread_manager->m_threadtable.get_ref(tid);
 
-		if(it != m_thread_manager->m_threadtable.end())
+		if(thr)
 		{
 	#ifdef GATHER_INTERNAL_STATS
 			m_thread_manager->m_non_cached_lookups->increment();
@@ -879,10 +926,10 @@ private:
 			if(!lookup_only)
 			{
 				m_thread_manager->m_last_tid = tid;
-				m_thread_manager->m_last_tinfo = &(it->second);
-				m_thread_manager->m_last_tinfo->m_lastaccess_ts = m_lastevent_ts;
+				m_thread_manager->m_last_tinfo = thr;
+				thr->m_lastaccess_ts = m_lastevent_ts;
 			}
-			return &(it->second);
+			return thr;
 		}
 		else
 		{
@@ -914,6 +961,8 @@ private:
 		scap_fseek(m_h, filepos);
 	}
 
+	void add_suppressed_comms(scap_open_args &oargs);
+
 	scap_t* m_h;
 	uint32_t m_nevts;
 	int64_t m_filesize;
@@ -924,6 +973,8 @@ private:
 	// <m_input_fd>". Otherwise, reading from m_input_filename.
 	int m_input_fd;
 	string m_input_filename;
+	bool m_bpf;
+	string m_bpf_probe;
 	bool m_isdebug_enabled;
 	bool m_isfatfile_enabled;
 	bool m_isinternal_events_enabled;
@@ -1012,10 +1063,10 @@ public:
 #ifdef GATHER_INTERNAL_STATS
 	sinsp_stats m_stats;
 #endif
-	uint32_t m_n_proc_lookups;
+	int32_t m_n_proc_lookups;
 	uint64_t m_n_proc_lookups_duration_ns;
-	uint32_t m_max_n_proc_lookups;
-	uint32_t m_max_n_proc_socket_lookups;
+	int32_t m_max_n_proc_lookups = -1;
+	int32_t m_max_n_proc_socket_lookups = -1;
 #ifdef HAS_ANALYZER
 	vector<uint64_t> m_tid_collisions;
 #endif
@@ -1099,12 +1150,17 @@ public:
 	uint64_t m_next_flush_time_ns;
 	uint64_t m_last_procrequest_tod;
 	sinsp_proc_metainfo m_meinfo;
+	uint64_t m_next_stats_print_time_ns;
 
 	static unsigned int m_num_possible_cpus;
 #if defined(HAS_CAPTURE)
 	int64_t m_sysdig_pid;
 #endif
 
+	// Any thread with a comm in this set will not have its events
+	// returned in sinsp::next()
+	std::set<std::string> m_suppressed_comms;
+
 	friend class sinsp_parser;
 	friend class sinsp_analyzer;
 	friend class sinsp_analyzer_parsers;
