@@ -1,19 +1,20 @@
 /*
-Copyright (C) 2013-2014 Draios inc.
+Copyright (C) 2013-2018 Draios Inc dba Sysdig.
 
 This file is part of sysdig.
 
-sysdig is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-sysdig is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 
-You should have received a copy of the GNU General Public License
-along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #pragma once
@@ -22,6 +23,15 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #define VISIBILITY_PRIVATE private:
 #endif
 
+#ifdef _WIN32
+struct iovec {
+	void  *iov_base;    /* Starting address */
+	size_t iov_len;     /* Number of bytes to transfer */
+};
+#else
+#include <sys/uio.h>
+#endif
+
 #include <functional>
 
 class sinsp_delays_info;
@@ -108,7 +118,8 @@ public:
 #ifndef _WIN32
 	inline sinsp_threadinfo* get_main_thread()
 	{
-		if(m_main_thread == NULL)
+		auto main_thread = m_main_thread.lock();
+		if(!main_thread)
 		{
 			//
 			// Is this a child thread?
@@ -129,17 +140,17 @@ public:
 				//
 				// Yes, this is a child thread. Find the process root thread.
 				//
-				sinsp_threadinfo* ptinfo = lookup_thread();
-				if(NULL == ptinfo)
+				auto ptinfo = lookup_thread();
+				if (!ptinfo)
 				{
 					return NULL;
 				}
-
 				m_main_thread = ptinfo;
+				return &*ptinfo;
 			}
 		}
 
-		return m_main_thread;
+		return &*main_thread;
 	}
 #else
 	sinsp_threadinfo* get_main_thread();
@@ -250,6 +261,7 @@ public:
 	size_t m_program_hash;
 	size_t m_program_hash_falco;
 	int32_t m_tty;
+	int32_t m_loginuid; ///< loginuid (auid)
 
 	//
 	// State for multi-event processing
@@ -267,6 +279,19 @@ public:
 
 	thread_analyzer_info* m_ainfo;
 
+	size_t args_len() const;
+	size_t env_len() const;
+	size_t cgroups_len() const;
+
+	void args_to_iovec(struct iovec **iov, int *iovcnt,
+			   std::string &rem) const;
+
+	void env_to_iovec(struct iovec **iov, int *iovcnt,
+			  std::string &rem) const;
+
+	void cgroups_to_iovec(struct iovec **iov, int *iovcnt,
+			      std::string &rem) const;
+
 #ifdef HAS_FILTERING
 	//
 	// State for filtering
@@ -311,6 +336,7 @@ VISIBILITY_PRIVATE
 	sinsp_threadinfo* get_cwd_root();
 	void set_args(const char* args, size_t len);
 	void set_env(const char* env, size_t len);
+	bool set_env_from_proc();
 	void set_cgroups(const char* cgroups, size_t len);
 	bool is_lastevent_data_valid();
 	inline void set_lastevent_data_validity(bool isvalid)
@@ -326,10 +352,19 @@ VISIBILITY_PRIVATE
 	}
 	void allocate_private_state();
 	void compute_program_hash();
-	sinsp_threadinfo* lookup_thread();
-	inline void args_to_scap(scap_threadinfo* sctinfo);
-	inline void env_to_scap(scap_threadinfo* sctinfo);
-	inline void cgroups_to_scap(scap_threadinfo* sctinfo);
+	shared_ptr<sinsp_threadinfo> lookup_thread();
+
+	size_t strvec_len(const vector<string> &strs) const;
+	void strvec_to_iovec(const vector<string> &strs,
+			     struct iovec **iov, int *iovcnt,
+			     std::string &rem) const;
+
+	void add_to_iovec(const string &str,
+			  const bool include_trailing_null,
+			  struct iovec &iov,
+			  uint32_t &alen,
+			  std::string &rem) const;
+
 	void fd_to_scap(scap_fdinfo *dst, sinsp_fdinfo_t* src);
 
 	//  void push_fdop(sinsp_fdop* op);
@@ -342,7 +377,7 @@ VISIBILITY_PRIVATE
 	//
 	sinsp_fdtable m_fdtable; // The fd table of this thread
 	string m_cwd; // current working directory
-	sinsp_threadinfo* m_main_thread;
+	weak_ptr<sinsp_threadinfo> m_main_thread;
 	uint8_t* m_lastevent_data; // Used by some event parsers to store the last enter event
 	vector<void*> m_private_state;
 
@@ -367,8 +402,67 @@ VISIBILITY_PRIVATE
 
 /*@}*/
 
-typedef unordered_map<int64_t, sinsp_threadinfo> threadinfo_map_t;
-typedef threadinfo_map_t::iterator threadinfo_map_iterator_t;
+class threadinfo_map_t
+{
+public:
+	typedef std::function<bool(sinsp_threadinfo&)> visitor_t;
+	typedef std::shared_ptr<sinsp_threadinfo> ptr_t;
+
+	inline void put(sinsp_threadinfo* tinfo)
+	{
+		m_threads[tinfo->m_tid] = ptr_t(tinfo);
+	}
+
+	inline sinsp_threadinfo* get(uint64_t tid)
+	{
+		auto it = m_threads.find(tid);
+		if (it == m_threads.end())
+		{
+			return  nullptr;
+		}
+		return it->second.get();
+	}
+
+	inline ptr_t get_ref(uint64_t tid)
+	{
+		auto it = m_threads.find(tid);
+		if (it == m_threads.end())
+		{
+			return  nullptr;
+		}
+		return it->second;
+	}
+
+	inline void erase(uint64_t tid)
+	{
+		m_threads.erase(tid);
+	}
+
+	inline void clear()
+	{
+		m_threads.clear();
+	}
+
+	bool loop(visitor_t callback)
+	{
+		for (auto& it : m_threads)
+		{
+			if (!callback(*it.second.get()))
+			{
+				return false;
+			}
+		}
+		return true;
+	}
+
+	inline size_t size() const
+	{
+		return m_threads.size();
+	}
+
+protected:
+	unordered_map<int64_t, ptr_t> m_threads;
+};
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -407,7 +501,7 @@ public:
 	void clear();
 
 	void set_listener(sinsp_threadtable_listener* listener);
-	void add_thread(sinsp_threadinfo& threadinfo, bool from_scap_proctable);
+	void add_thread(sinsp_threadinfo* threadinfo, bool from_scap_proctable);
 	void remove_thread(int64_t tid, bool force);
 	// Returns true if the table is actually scanned
 	// NOTE: this is implemented in sinsp.cpp so we can inline it from there
@@ -434,19 +528,17 @@ public:
 	set<uint16_t> m_server_ports;
 
 private:
-	void remove_thread(threadinfo_map_iterator_t it, bool force);
 	void increment_mainthread_childcount(sinsp_threadinfo* threadinfo);
-	inline void clear_thread_pointers(threadinfo_map_iterator_t it);
+	inline void clear_thread_pointers(sinsp_threadinfo& threadinfo);
 	void free_dump_fdinfos(vector<scap_fdinfo*>* fdinfos_to_free);
 	void thread_to_scap(sinsp_threadinfo& tinfo, scap_threadinfo* sctinfo);
 
 	sinsp* m_inspector;
 	threadinfo_map_t m_threadtable;
 	int64_t m_last_tid;
-	sinsp_threadinfo* m_last_tinfo;
+	std::weak_ptr<sinsp_threadinfo> m_last_tinfo;
 	uint64_t m_last_flush_time_ns;
 	uint32_t m_n_drops;
-	uint32_t m_n_proc_lookups;
 
 	sinsp_threadtable_listener* m_listener;
 
