@@ -1,17 +1,20 @@
 --[[
-Copyright (C) 2017 Draios inc.
+Copyright (C) 2013-2018 Draios Inc dba Sysdig.
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
+This file is part of sysdig.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
 --]]
 
 -- Chisel description
@@ -757,7 +760,7 @@ function on_interval(ts_s, ts_ns, delta)
 	if nintervals % percent_update_sample_period == 0 then
 		local progress = sysdig.get_read_progress()
 		if progress == 100 then
-			progress = 99
+			progress = 99.99
 		end
 		print('{"progress": ' .. progress .. ' },')
 		io.flush(stdout)
@@ -947,7 +950,7 @@ function build_output(captureDuration)
 	if should_include(gsummary.fileCountW) then
 		res[#res+1] = {
 			name = 'Modified Files',
-			desc = 'Number of files that have been accessed during the capture',
+			desc = 'Number of files that have been received writes during the capture',
 			category = 'file',
 			targetView = 'files',
 			drillDownKey = 'fd.directory',
@@ -1162,7 +1165,7 @@ function build_output(captureDuration)
 			category = 'performance',
 			targetView = 'echo',
 			targetViewTitle = 'HTTP responses',
-			targetViewFilter = 'fd.type=ipv4 and evt.buffer contains "HTTP/"',
+			targetViewFilter = '(fd.type=ipv4 or fd.type=ipv6) and evt.buffer contains "HTTP/"',
 			drillDownKey = 'fd.directory',
 			data = gsummary.sysReqCountHttp
 		}
@@ -1175,7 +1178,7 @@ function build_output(captureDuration)
 			category = 'performance',
 			targetView = 'echo',
 			targetViewTitle = 'HTTP responses',
-			targetViewFilter = 'fd.type=ipv4 and evt.arg.data startswith "HTTP/" and not evt.arg.data contains "200"',
+			targetViewFilter = '(fd.type=ipv4 or fd.type=ipv6) and evt.arg.data startswith "HTTP/" and not evt.arg.data contains "200"',
 			drillDownKey = 'fd.directory',
 			data = gsummary.sysErrCountHttp
 		}
